//---------------------------------------------------------------------------
// TestAbsoluteGrowth
//---------------------------------------------------------------------------
#if !defined(TestAbsoluteGrowth_H)
  #define TestAbsoluteGrowth_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clSimManager;

/**
* Tests the clAbsoluteGrowth class.  This tests all six types of absolute
* growth - unlimited absolute growth, absolute growth limited to constant
* radial increment, absolute growth limited to constant basal area increment,
* unlimited absolute growth - diameter update only, absolute growth limited to
* constant radial increment - diameter update only, and absolute growth limited
* to constant basal area increment - diameter update only.
*
* I have not yet put in testing of suppression/release.
*/
class clTestAbsoluteGrowth : public CppUnit::TestFixture {

  public:

  /**
  * Combines the testing into one behavior.
  */
  void TestAbsoluteGrowth();

  protected:

  /**
  * Tests the growth when the timestep length is one year.  WriteXMLFile1()
  * is used to set up the run.  10 trees are created of three species,
  * assigned to all six growth types and with growth levels both above and
  * below limits.  GLIs are assigned to them, and then growth is run.  Their
  * diameters, heights, and lgm values are checked to make sure they are
  * correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test1YearPerTimestep(clSimManager *p_oSimManager);

  /**
  * Tests the growth when the timestep length is three years.  WriteXMLFile1()
  * is used to set up the run.  10 trees are created of three species,
  * assigned to all six growth types and with growth levels both above and
  * below limits.  GLIs are assigned to them, and then growth is run.  Their
  * diameters, heights, and lgm values are checked to make sure they are
  * correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test3YearsPerTimestep(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to direct testing.  Timestep length is 1 year.
  * This contains three species.
  * Behaviors:
  * <ul>
  * <li>Unlimited absolute growth is applied to Species 1 saplings.
  * <li>Absolute growth limited to constant radial increment is applied to
  * Species 1 adults.
  * <li>Absolute growth limited to basal area increment is applied to Species
  * 2 adults.
  * <li>Unlimited absolute growth - diameter only is applied to Species 2
  * saplings.
  * <li>Absolute growth limited to constant radial increment - diameter only is
  * applied to Species 3 saplings.
  * <li>Absolute growth limited to basal area increment - diameter only is
  * applied to Species 3 adults.
  * Height incrementer applied to all with diameter only behaviors.
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a parameter file to direct testing.  Timestep length is 3 years.
  * This contains three species.
  * Behaviors:
  * <ul>
  * <li>Unlimited absolute growth is applied to Species 1 saplings.
  * <li>Absolute growth limited to constant radial increment is applied to
  * Species 1 adults.
  * <li>Absolute growth limited to basal area increment is applied to Species
  * 2 adults.
  * <li>Unlimited absolute growth - diameter only is applied to Species 2
  * saplings.
  * <li>Absolute growth limited to constant radial increment - diameter only is
  * applied to Species 3 saplings.
  * <li>Absolute growth limited to basal area increment - diameter only is
  * applied to Species 3 adults.
  * Height incrementer applied to all with diameter only behaviors.
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile2();

};
//---------------------------------------------------------------------------
#endif // TestAbsoluteGrowth_H
