//---------------------------------------------------------------------------
// TestCarbonValueCalculator
//---------------------------------------------------------------------------
#if !defined(TestCarbonValueCalculator_H)
  #define TestCarbonValueCalculator_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;
class clTreePopulation;
class clTree;

/**
* Tests the class clCarbonValueCalculator.
*/
class clTestCarbonValueCalculator : public CppUnit::TestFixture {

  public:

  /**
  * Performs all the tests from one function.
  */
  void TestCarbonValueCalculator();

  protected:

  /**
  * Tests normal processing.  WriteXMLFile1() does setup.  This function 
  * creates three trees for each species, of varying sizes.  The run goes for a
  * timestep and then the function checks the carbon and carbon value for each
  * species.  Then this kills all trees and runs another timestep to check
  * grid reset.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestNormalProcessing(clSimManager *p_oSimManager);

  /**
  * Tests error processing.  This uses WriteXMLErrorFileX() to introduce errors
  * and makes sure that they are caught.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes the parameter file for normal processing.  There are 19 species 
  * (Species 13 is not used).  The biomass calculator is the only behavior and 
  * is applied to saplings and adults of all used species.  All 18 used 
  * species use dimension analysis equation 1.  The first 9 use a correction 
  * factor; the second 9 don't.
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes an error file where the Dimension Analysis behavior is not enabled
  * for all trees for carbon value.
  * @return Filename written.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes an error file where the percentage of biomass that is carbon is 
  * negative.
  * @return Filename written.
  */
  char* WriteXMLErrorFile2();

  /**
  * Writes an error file where the percentage of biomass that is carbon is 
  * more than 100.
  * @return Filename written.
  */
  char* WriteXMLErrorFile3();

  /**
  * Writes common portions of the test parameter files (tree pop, plot, etc).
  * This ends with &lt;/behaviorList&gt.  There are only two species for this
  * file.
  * @param oOut File stream to write to.
  */
  void WriteCommonErrorStuff(std::fstream &oOut);
};
//---------------------------------------------------------------------------
#endif // TestBiomassCalculator_H
