//---------------------------------------------------------------------------
// TestCompetitionHarvest
//---------------------------------------------------------------------------
#if !defined(TestCompetitionHarvest_H)
  #define TestCompetitionHarvest_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;
class clTreePopulation;

/**
* Tests the clCompetitionHarvest class.
*/
class clTestCompetitionHarvest : public CppUnit::TestFixture
{
  public:

  /**
  * Groups together all the other harvest testing functions into a comprehensive
  * harvest test.
  */
  void TestCompetitionHarvest();

  protected:

  /**
  * Performs the first harvest run.
  * @param p_oSimManager Sim manager object.
  */
  void DoHarvestRun1(clSimManager *p_oSimManager);

  /**
  * Performs the second harvest run.
  * @param p_oSimManager Sim manager object.
  */
  void DoHarvestRun2(clSimManager *p_oSimManager);

/**
  * Performs the third harvest run.
  * @param p_oSimManager Sim manager object.
  */
  void DoHarvestRun3(clSimManager *p_oSimManager);
  
  /**
   * Tests error processing.
   * @param p_oSimManager Sim manager object.
   */
  void DoErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file for run 1.
  * @return Filename written.
  */
  char * WriteXMLFile1();
  
  /**
  * Writes a parameter file for run 1, without the file written.
  * @return Filename written.
  */
  char * WriteXMLFile1a();

  /**
  * Writes a parameter file for run 2.
  * @return Filename written.
  */
  char * WriteXMLFile2();

  /**
  * Writes a parameter file for run 3.
  * @return Filename written.
  */
  char * WriteXMLFile3();
  
  /**
  * Writes a parameter file where the competition harvest behavior is applied 
  * to seedlings.
  * @return Filename written.
  */
  char * WriteXMLErrorFile1();
  
  /**
  * Writes a parameter file where the harvest type is fixed interval and the
  * harvest interval is less than 1.
  * @return Filename written.
  */
  char * WriteXMLErrorFile2();
  
  /**
  * Writes a parameter file where the harvest type is fixed BA with proportion
  * to cut but cut amount is not a proportion between 0 and 1.
  * @return Filename written.
  */
  char * WriteXMLErrorFile3();
  
  /**
  * Writes a parameter file where the species proportions to cut do not add up 
  * to 1.
  * @return Filename written.
  */
  char * WriteXMLErrorFile4();
  
  /**
  * Writes a parameter file where the harvest type is fixed BA but the minimum
  * interval is less than 1.
  * @return Filename written.
  */
  char * WriteXMLErrorFile5();
  
  /**
  * Writes a parameter file where a value in max crowding radius is not 
  * greater than 0.
  * @return Filename written.
  */
  char * WriteXMLErrorFile6();
  
  /**
  * Writes common portions of the test parameter files (tree pop, plot, etc).
  * Ends with &lt;/allometry&gt;.
  * @param oOut File stream to write to.
  */
  void WriteCommonErrorStuff(std::fstream &oOut);

};
//---------------------------------------------------------------------------
#endif // TestCompetitionHarvest_H
