//---------------------------------------------------------------------------
// TestCrownRadiusReporter.cpp
//---------------------------------------------------------------------------
#include "TestCrownRadiusReporter.h"
#include <fstream>
#include <math.h>
#include "TreePopulation.h"
#include "SimManager.h"
#include "BehaviorBase.h"

////////////////////////////////////////////////////////////////////////////
//  TestCrownRadiusReporter()
////////////////////////////////////////////////////////////////////////////
void clTestCrownRadiusReporter::TestCrownRadiusReporter() {
  clSimManager *p_oSimManager = new clSimManager(4, 2, "");

  TestNormalProcessingRun1(p_oSimManager);
  TestErrorProcessing(p_oSimManager);

  delete p_oSimManager;
}

////////////////////////////////////////////////////////////////////////////
//  TestNormalProcessingRun1()
////////////////////////////////////////////////////////////////////////////
void clTestCrownRadiusReporter::TestNormalProcessingRun1(clSimManager *p_oSimManager) {
  clTreePopulation * p_oPop;
  clTree *p_oTargetTree1, *p_oTargetTree2, *p_oTargetTree3,
         *p_oTargetTree4, *p_oTargetTree5, *p_oTargetTree6,
         *p_oTargetTree7, *p_oTargetTree8, *p_oTargetTree9,
         *p_oTargetTree10, *p_oTargetTree11, *p_oTargetTree12,
         *p_oTargetTree13, *p_oTargetTree14, *p_oTargetTree15;
  float fExpectedRad, fActualRad, fDbh;
  int iSpecies;

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile1() );
  p_oPop = ( clTreePopulation * ) p_oSimManager->GetPopulationObject( "treepopulation" );

  //Create our trees
  iSpecies = 0;
  p_oTargetTree1 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::snag, 12);
  p_oTargetTree2 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::sapling, 5);
  p_oTargetTree3 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, 15);

  iSpecies = 1;
  p_oTargetTree4 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::snag, 20);
  p_oTargetTree5 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::sapling, 6);
  p_oTargetTree6 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, 25);

  p_oTargetTree7 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::snag, 30);
  p_oTargetTree8 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::sapling, 7);
  p_oTargetTree9 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, 35);

  iSpecies = 2;
  p_oTargetTree10 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::snag, 40);
  p_oTargetTree11 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::sapling, 4);
  p_oTargetTree12 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, 45);
  
  p_oTargetTree13 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::snag, 50);
  p_oTargetTree14 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::sapling, 5);
  p_oTargetTree15 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, 55);

  //Run the crown radius behavior only
  int i;
  for (i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    clBehaviorBase *p_oBase = p_oSimManager->GetBehaviorObject(i);
    p_oBase->Action();
  }

  //Check the radius for each tree
  CPPUNIT_ASSERT(-1 == p_oPop->GetFloatDataCode("Crown_Radius", p_oTargetTree1->GetSpecies(),  p_oTargetTree1->GetType()));
  CPPUNIT_ASSERT(-1 == p_oPop->GetFloatDataCode("Crown_Radius", p_oTargetTree2->GetSpecies(),  p_oTargetTree2->GetType()));
  CPPUNIT_ASSERT(-1 == p_oPop->GetFloatDataCode("Crown_Radius", p_oTargetTree3->GetSpecies(),  p_oTargetTree3->GetType()));

  fExpectedRad = 6.400795603;
  p_oTargetTree4->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree4->GetSpecies(), p_oTargetTree4->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 0.3684;
  p_oTargetTree5->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree5->GetSpecies(), p_oTargetTree5->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 7.069146174;
  p_oTargetTree6->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree6->GetSpecies(), p_oTargetTree6->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);

  fExpectedRad = 7.507239318;
  p_oTargetTree7->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree7->GetSpecies(), p_oTargetTree7->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 0.4298;
  p_oTargetTree8->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree8->GetSpecies(), p_oTargetTree8->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 7.791323383;
  p_oTargetTree9->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree9->GetSpecies(), p_oTargetTree9->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);

  fExpectedRad = 1.398692632;
  p_oTargetTree10->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree10->GetSpecies(), p_oTargetTree10->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 0.888112734;
  p_oTargetTree11->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree11->GetSpecies(), p_oTargetTree11->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 1.592172292;
  p_oTargetTree12->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree12->GetSpecies(), p_oTargetTree12->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);

  fExpectedRad = 1.787817983;
  p_oTargetTree13->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree13->GetSpecies(), p_oTargetTree13->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 1.146249741;
  p_oTargetTree14->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree14->GetSpecies(), p_oTargetTree14->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 1.985433087;
  p_oTargetTree15->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree15->GetSpecies(), p_oTargetTree15->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);

  //Run another timestep
  p_oSimManager->RunSim(1);

  //Check the radius for each tree
  CPPUNIT_ASSERT(-1 == p_oPop->GetFloatDataCode("Crown_Radius", p_oTargetTree1->GetSpecies(),  p_oTargetTree1->GetType()));
  CPPUNIT_ASSERT(-1 == p_oPop->GetFloatDataCode("Crown_Radius", p_oTargetTree2->GetSpecies(),  p_oTargetTree2->GetType()));
  CPPUNIT_ASSERT(-1 == p_oPop->GetFloatDataCode("Crown_Radius", p_oTargetTree3->GetSpecies(),  p_oTargetTree3->GetType()));

  fExpectedRad = 6.400795603;
  p_oTargetTree4->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree4->GetSpecies(), p_oTargetTree4->GetType()), &fActualRad);
  p_oTargetTree4->GetValue(p_oPop->GetDbhCode(p_oTargetTree4->GetSpecies(), p_oTargetTree4->GetType()), &fDbh);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  CPPUNIT_ASSERT(fabs(fDbh - 20) < 0.001);
  
  fExpectedRad = 0.51576;
  p_oTargetTree5->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree5->GetSpecies(), p_oTargetTree5->GetType()), &fActualRad);
  p_oTargetTree5->GetValue(p_oPop->GetDbhCode(p_oTargetTree5->GetSpecies(), p_oTargetTree5->GetType()), &fDbh);
  CPPUNIT_ASSERT(fabs(fDbh - 8.4) < 0.001);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  
  fExpectedRad = 7.302866109;
  p_oTargetTree6->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree6->GetSpecies(), p_oTargetTree6->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);

  fExpectedRad = 7.507239318;
  p_oTargetTree7->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree7->GetSpecies(), p_oTargetTree7->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 0.57716;
  p_oTargetTree8->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree8->GetSpecies(), p_oTargetTree8->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 7.88926763;
  p_oTargetTree9->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree9->GetSpecies(), p_oTargetTree9->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);

  fExpectedRad = 1.398692632;
  p_oTargetTree10->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree10->GetSpecies(), p_oTargetTree10->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 1.684517695;
  p_oTargetTree11->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree11->GetSpecies(), p_oTargetTree11->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 1.701480526;
  p_oTargetTree12->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree12->GetSpecies(), p_oTargetTree12->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);

  fExpectedRad = 1.787817983;
  p_oTargetTree13->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree13->GetSpecies(), p_oTargetTree13->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 2.006627679;
  p_oTargetTree14->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree14->GetSpecies(), p_oTargetTree14->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
  fExpectedRad = 2.096896133;
  p_oTargetTree15->GetValue(p_oPop->GetFloatDataCode("Crown_Radius",  p_oTargetTree15->GetSpecies(), p_oTargetTree15->GetType()), &fActualRad);
  CPPUNIT_ASSERT(fabs(fActualRad - fExpectedRad) < 0.001);
}

////////////////////////////////////////////////////////////////////////////
//  TestErrorProcessing()
////////////////////////////////////////////////////////////////////////////
void clTestCrownRadiusReporter::TestErrorProcessing(clSimManager *p_oSimManager) {

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile1());
    CppUnit::Asserter::fail("TestCrownRadiusReporter error processing didn't catch error for WriteXMLErrorFile1.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "RegisterTreeDataMembers") == NULL)
    {
      CppUnit::Asserter::fail("TestCrownRadiusReporter error processing caught wrong error for WriteXMLErrorFile1.");
    }
    ;
  }
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
////////////////////////////////////////////////////////////////////////////
char* clTestCrownRadiusReporter::WriteXMLFile1() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>3</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\" />"
       << "<tr_species speciesName=\"Species_2\" />"
       << "<tr_species speciesName=\"Species_3\" />"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>";

  oOut << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">45</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0614</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_3\">0.02418</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">0.9</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_3\">1.1</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">1</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">1</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">1</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">1</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_3\">-0.2</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.368</tr_sachVal>"
       << "<tr_sachVal species=\"Species_3\">0.34</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">0.5</tr_scheVal>"
       << "<tr_scheVal species=\"Species_3\">0.9</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.2871</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">1</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_3\">1</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "<tr_chRichCrownRadIntercept>"
       << "<tr_crcriVal species=\"Species_2\">0.3</tr_crcriVal>"
       << "<tr_crcriVal species=\"Species_3\">0.4</tr_crcriVal>"
       << "</tr_chRichCrownRadIntercept>"
       << "<tr_chRichCrownRadAsymp>"
       << "<tr_crcraVal species=\"Species_2\">8</tr_crcraVal>"
       << "<tr_crcraVal species=\"Species_3\">7</tr_crcraVal>"
       << "</tr_chRichCrownRadAsymp>"
       << "<tr_chRichCrownRadShape1b>"
       << "<tr_crcrs1bVal species=\"Species_2\">0.09</tr_crcrs1bVal>"
       << "<tr_crcrs1bVal species=\"Species_3\">0.1</tr_crcrs1bVal>"
       << "</tr_chRichCrownRadShape1b>"
       << "<tr_chRichCrownRadShape2c>"
       << "<tr_crcrs2cVal species=\"Species_2\">1.5</tr_crcrs2cVal>"
       << "<tr_crcrs2cVal species=\"Species_3\">2.4</tr_crcrs2cVal>"
       << "</tr_chRichCrownRadShape2c>"
       << "<tr_chRichCrownHtIntercept>"
       << "<tr_crchiVal species=\"Species_2\">0.6</tr_crchiVal>"
       << "<tr_crchiVal species=\"Species_3\">0.5</tr_crchiVal>"
       << "</tr_chRichCrownHtIntercept>"
       << "</allometry>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>constradialgrowth</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>Crown Radius Reporter</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Snag\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<growth>"
       << "<gr_adultConstRadialInc>"
       << "<gr_acriVal species=\"Species_1\">10</gr_acriVal>"
       << "<gr_acriVal species=\"Species_2\">12</gr_acriVal>"
       << "<gr_acriVal species=\"Species_3\">14</gr_acriVal>"
       << "</gr_adultConstRadialInc>"
       << "</growth>"
       << "</paramFile>";

  oOut.close();
  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile1()
////////////////////////////////////////////////////////////////////////////
char* clTestCrownRadiusReporter::WriteXMLErrorFile1() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>3</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\" />"
       << "<tr_species speciesName=\"Species_2\" />"
       << "<tr_species speciesName=\"Species_3\" />"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>";

  oOut << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">45</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0614</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_3\">0.02418</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">0.9</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_3\">1.1</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">1</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_3\">-0.2</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.368</tr_sachVal>"
       << "<tr_sachVal species=\"Species_3\">0.34</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">0.5</tr_scheVal>"
       << "<tr_scheVal species=\"Species_3\">0.9</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.2871</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">1</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_3\">1</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "<tr_chRichCrownRadIntercept>"
       << "<tr_crcriVal species=\"Species_2\">0.3</tr_crcriVal>"
       << "<tr_crcriVal species=\"Species_3\">0.4</tr_crcriVal>"
       << "</tr_chRichCrownRadIntercept>"
       << "<tr_chRichCrownRadAsymp>"
       << "<tr_crcraVal species=\"Species_2\">8</tr_crcraVal>"
       << "<tr_crcraVal species=\"Species_3\">7</tr_crcraVal>"
       << "</tr_chRichCrownRadAsymp>"
       << "<tr_chRichCrownRadShape1b>"
       << "<tr_crcrs1bVal species=\"Species_2\">0.09</tr_crcrs1bVal>"
       << "<tr_crcrs1bVal species=\"Species_3\">0.1</tr_crcrs1bVal>"
       << "</tr_chRichCrownRadShape1b>"
       << "<tr_chRichCrownRadShape2c>"
       << "<tr_crcrs2cVal species=\"Species_2\">1.5</tr_crcrs2cVal>"
       << "<tr_crcrs2cVal species=\"Species_3\">2.4</tr_crcrs2cVal>"
       << "</tr_chRichCrownRadShape2c>"
       << "<tr_chRichCrownHtIntercept>"
       << "<tr_crchiVal species=\"Species_2\">0.6</tr_crchiVal>"
       << "<tr_crchiVal species=\"Species_3\">0.5</tr_crchiVal>"
       << "</tr_chRichCrownHtIntercept>"
       << "</allometry>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>constradialgrowth</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Snag\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>Crown Radius Reporter</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Snag\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "</paramFile>";

  oOut.close();
  return cFileString;
}
