//---------------------------------------------------------------------------
// TestCrownRadiusReporter
//---------------------------------------------------------------------------
#if !defined(TestCrownRadiusReporter_H)
  #define TestCrownRadiusReporter_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the class clCrownRadiusReporter.
*/
class clTestCrownRadiusReporter : public CppUnit::TestFixture {

  public:

  /**
  * Performs all the tests from one function.
  */
  void TestCrownRadiusReporter();

  protected:

  /**
  * Tests normal processing, run 1.  WriteXMLFile1() does setup.  Several
  * trees are created and their radii checked to make sure it's correct after
  * each of two timesteps.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestNormalProcessingRun1(clSimManager *p_oSimManager);

  /**
  * Tests error processing.  This uses WriteXMLErrorFile1() to introduce errors
  * and makes sure that they are caught.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes the parameter file for normal processing.  There are 3 species;
  * timestep length is 1 year; there are two behaviors, growth and the crown
  * radius reporter behavior.
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes an error file where the behavior is applied to seedlings.
  */
  char* WriteXMLErrorFile1();
};
//---------------------------------------------------------------------------
#endif // TestCrownRadiusReporter_H
