//---------------------------------------------------------------------------
// TestFuncResponseSeedPredation
//---------------------------------------------------------------------------
#if !defined(TestFuncResponseSeedPredation_H)
  #define TestFuncResponseSeedPredation_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clGridBase;
class clSimManager;

/**
 * Tests seed predation. This can't do a thorough test - this will test error
 * processing and cause an output file to be printed of seed model results
 * so the output can be checked manually.
 */
class clTestFuncResponseSeedPredation : public CppUnit::TestFixture {

  public:

  /**
  * Rolls up all the seed predation tests.
  */
  void TestFuncResponseSeedPredation();

  protected:
  /**
  * Tests error processing for bad input.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Creates a controlled run.  Writes an XML file (WriteXMLFile1) and runs it
  * one timestep to produce output.
  */
  void TestSeedPredationNormal(clSimManager *p_oSimManager);

  /**
  * Tests setup under normal processing conditions for WriteXMLFile2.  This
  * makes sure that maps are treated correctly and initial densities set up right.
  */
  void TestSetup1(clSimManager *p_oSimManager);

  /**
  * Tests setup under normal processing conditions for WriteXMLFile3.  This
  * makes sure that maps are treated correctly and initial densities set up right.
  */
  void TestSetup2(clSimManager *p_oSimManager);

  /**
  * Tests setup under normal processing conditions for WriteXMLFile4.  This
  * makes sure that maps are treated correctly and initial densities set up right.
  */
  void TestSetup3(clSimManager *p_oSimManager);

  /**
  * Tests setup under normal processing conditions for WriteXMLFile5.  This
  * makes sure that maps are treated correctly and initial densities set up right.
  */
  void TestSetup4(clSimManager *p_oSimManager);

  /**
  * This verifies that densities are correctly written each timestep.  If the
  * predator densities are to be preserved, the values in the predator grid
  * will be different after a second timestep; if they are not, it will be
  * the same (assuming the seed grid input is the same).  WriteXMLFile6() will
  * create the file with two timesteps where the densities are to be preserved;
  * WriteXMLFile7() is the same but densities are not to be preserved.
  */
  void TestDensities(clSimManager *p_oSimManager);

  /**
  * This verifies that the correct number of seeds is left in the seed grid
  * after the model runs.  This is based on Wendy's spreadsheet - how many
  * should be left if you subtract all the offtake from the original number.
  * This will use WriteXMLFile1().
  */
  void TestSeedOfftake1(clSimManager *p_oSimManager);
  
  /**
  * This tests the combination of regular and linked seed predation.
  * This will use WriteXMLOfftakeFile2().
  */
  void TestSeedOfftake2(clSimManager *p_oSimManager);
  
  /**
  * This tests seed pooling across cells and species for linked seed predation.
  * This will use WriteXMLOfftakeFile3().
  */
  void TestSeedOfftake3(clSimManager *p_oSimManager);
  
  /**
  * This tests species- and season-specific parameters.
  * This will use WriteXMLOfftakeFile4().
  */
  void TestSeedOfftake4(clSimManager *p_oSimManager);

  /**
  * Writes an XML file that will create the sim manager so we can create a
  * grid through the proper channels.  Characteristics:
  * <ul>
  * <li>Plot size of 160 m by 160 m.
  * <li>One species.
  * <li>Creates the seed grid with 1 grid cell the size of the plot.
  * <li>Predator initial density:  0.00046875
  * <li>Number of seeds:  51200000
  * <li>Max instantaneous decline rate:  -0.050622442
  * <li>Demographic efficiency:  0.000154258
  * <li>Density-dependent coefficient:  -0.000718667
  * <li>Maximum weekly intake:  7295.4
  * <li>Foraging efficiency:  0.001389
  * <li>Seedfall: 12 weeks.
  * <li>Model run:  52 weeks.
  * </ul>
  * @return Filename just written.
  */
  char* WriteXMLFile1();

  /**
  * Writes an XML file that will create the sim manager so we can create a
  * grid through the proper channels.  Characteristics:
  * <ul>
  * <li>Plot size of 160 m by 160 m.
  * <li>One species.
  * <li>Disperse behavior is present.
  * <li>Predator initial density:  0.46875
  * <li>Number of seeds:  51200000
  * <li>Max instantaneous decline rate:  -0.050622442
  * <li>Demographic efficiency:  0.000154258
  * <li>Density-dependent coefficient:  -0.000718667
  * <li>Maximum weekly intake:  7295.4
  * <li>Foraging efficiency:  0.001389
  * <li>Seedfall: 12 weeks.
  * <li>Model run:  52 weeks.
  * <li>Preserve predator density:  true.
  * </ul>
  * @return Filename just written.
  */
  char* WriteXMLFile2();

  /**
  * Writes an XML file that will create the sim manager so we can create a
  * grid through the proper channels.  Characteristics:
  * <ul>
  * <li>Plot size of 160 m by 160 m.
  * <li>One species.
  * <li>Disperse behavior is present.
  * <li>Predator initial density:  0.00046875
  * <li>Number of seeds:  51200000
  * <li>Max instantaneous decline rate:  -0.050622442
  * <li>Demographic efficiency:  0.000154258
  * <li>Density-dependent coefficient:  -0.000718667
  * <li>Maximum weekly intake:  7295.4
  * <li>Foraging efficiency:  0.001389
  * <li>Seedfall: 12 weeks.
  * <li>Model run:  52 weeks.
  * <li>Preserve predator density:  false.
  * </ul>
  * @return Filename just written.
  */
  char* WriteXMLFile3();

  /**
  * Writes an XML file that will create the sim manager so we can create a
  * grid through the proper channels.  Characteristics:
  * <ul>
  * <li>Plot size of 160 m by 160 m.
  * <li>One species.
  * <li>Disperse behavior is present.
  * <li>Predator initial density:  0.00046875
  * <li>Number of seeds:  51200000
  * <li>Max instantaneous decline rate:  -0.050622442
  * <li>Demographic efficiency:  0.000154258
  * <li>Density-dependent coefficient:  -0.000718667
  * <li>Maximum weekly intake:  7295.4
  * <li>Foraging efficiency:  0.001389
  * <li>Seedfall: 12 weeks.
  * <li>Model run:  52 weeks.
  * <li>Preserve predator density:  true.
  * <li>Seed grid map present, cells are 2 X 4
  * <li>Predator map present, resolution same as seed grid, values in each
  * cell = X + Y
  * </ul>
  * @return Filename just written.
  */
  char* WriteXMLFile4();

  /**
  * Writes a file with two timesteps where predator densities are to be
  * preserved.
  * @return Filename just written.
  */
  char* WriteXMLFile6();

  /**
  * Writes a file with two timesteps where predator densities are not to be
  * preserved.
  * @return Filename just written.
  */
  char* WriteXMLFile7();

  /**
  * Writes a file with multiple species, including one that is not applied to
  * seed predation.
  * @return Filename just written.
  */
  char* WriteXMLFile8();
  
  /**
  * Writes a file with multiple species, using both regular and linked seed
  * predation.
  * @return Filename just written.
  */
  char* WriteXMLOfftakeTestFile2();
  
  /**
   * Writes a file with multiple species, using linked seed predation.
   * @return Filename just written.
   */
  char* WriteXMLOfftakeTestFile3();
  
  /**
   * Writes a file with multiple species, using species- and season-specific
   * parameters.
   * @return Filename just written.
   */
  char* WriteXMLOfftakeTestFile4();

  /**
  * Creates file that has a predator initial density of 0.
  */
  char* WriteXMLErrorFile1();

  /**
  * Creates file that has no disperse behaviors.
  */
  char* WriteXMLErrorFile2();

  /**
  * Creates file that has maps for seed grid and seed predators, and X
  * resolutions don't match.
  */
  char* WriteXMLErrorFile3();

  /**
  * Creates file that has weeks of seedfall = 0.
  */
  char* WriteXMLErrorFile4();

  /**
  * Creates file that has weeks of model = 0.
  */
  char* WriteXMLErrorFile5();

  /**
  * Creates file that has maps for seed grid and seed predators, and Y
  * resolutions don't match.
  */
  char* WriteXMLErrorFile6();

 /**
 * Creates file that has a map for seed predators, but is missing the
 * data member "num preds".
 */
  char* WriteXMLErrorFile7();

  /**
  * Creates file where week germination starts is < 0.
  */
  char* WriteXMLErrorFile8();

  /**
  * Creates file where week germination starts is > 52.
  */
  char* WriteXMLErrorFile9();

  /**
  * Creates file where proportion germinating is < 0.
  */
  char* WriteXMLErrorFile10();

  /**
  * Creates file where proportion germinating is > 1.
  */
  char* WriteXMLErrorFile11();
};
//---------------------------------------------------------------------------
#endif // TestFuncResponseSeedPredation_H
