//---------------------------------------------------------------------------
// TestLogisticBiLevelMortality.cpp
//---------------------------------------------------------------------------
#include "TestLogisticBiLevelMortality.h"
#include "SimManager.h"
#include "TreePopulation.h"
#include "GridBase.h"
#include "BehaviorBase.h"
#include <fstream.h>
#include <math.h>

////////////////////////////////////////////////////////////////////////////
// TestLogisticBiLevelMortality()
////////////////////////////////////////////////////////////////////////////
void clTestLogisticBiLevelMortality::TestLogisticBiLevelMortality() {
  clSimManager *p_oSimManager = new clSimManager(4, 2, "");

  NormalProcessingRun1(p_oSimManager);
  NormalProcessingRun2(p_oSimManager);
  NormalProcessingRun3(p_oSimManager);
  ErrorProcessing(p_oSimManager);

  delete p_oSimManager;
}

/////////////////////////////////////////////////////////////////////////////
//  NormalProcessingRun1()
/////////////////////////////////////////////////////////////////////////////
void clTestLogisticBiLevelMortality::NormalProcessingRun1(
    clSimManager *p_oSimManager) {
  clTreePopulation * p_oPop;
  clGridBase *p_oStormLight;
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  float fDiam, fY, fExpected;
  int *p_iNumTrees = new int[27], i, iSpecies, iType;

  for (i = 0; i < 27; i++) {
    p_iNumTrees[i] = 0;
  }

  p_oSimManager->ReadFile(WriteXMLFile1());
  p_oPop = (clTreePopulation *) p_oSimManager->GetPopulationObject(
      "treepopulation");
  p_oStormLight = p_oSimManager->GetGridObject("Storm Light");

  //Set our grid values
  fDiam = 15;
  p_oStormLight->SetValueOfCell(0, 0, p_oStormLight->GetFloatDataCode("Light"),
      fDiam);
  fDiam = 35;
  p_oStormLight->SetValueOfCell(0, 1, p_oStormLight->GetFloatDataCode("Light"),
      fDiam);
  fDiam = 80;
  p_oStormLight->SetValueOfCell(0, 2, p_oStormLight->GetFloatDataCode("Light"),
      fDiam);

  //Add 1000 trees to the plot of each kind
  for (i = 0; i < 1000; i++) {
    fDiam = 0.1;
    iSpecies = 0;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 0.2;
    iSpecies = 1;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 0.3;
    iSpecies = 3;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 2.5;
    iSpecies = 0;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 3.4;
    iSpecies = 1;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 5.4;
    iSpecies = 3;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 12.6;
    iSpecies = 0;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 30;
    iSpecies = 1;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 23.3;
    iSpecies = 3;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 0.1;
    iSpecies = 0;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 0.2;
    iSpecies = 1;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 0.3;
    iSpecies = 3;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 2.5;
    iSpecies = 0;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 3.4;
    iSpecies = 1;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 5.4;
    iSpecies = 3;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 12.6;
    iSpecies = 0;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 30;
    iSpecies = 1;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 23.3;
    iSpecies = 3;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 0.1;
    iSpecies = 0;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 0.2;
    iSpecies = 1;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 0.3;
    iSpecies = 3;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 2.5;
    iSpecies = 0;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 3.4;
    iSpecies = 1;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 5.4;
    iSpecies = 3;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 12.6;
    iSpecies = 0;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 30;
    iSpecies = 1;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 23.3;
    iSpecies = 3;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);
  }

  //Now run mortality, but skip storm light
  for (i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    clBehaviorBase *p_oBase = p_oSimManager->GetBehaviorObject(i);
    p_oBase->Action();
  }

  //Count up the trees
  p_oAllTrees = p_oPop->Find("all");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    iSpecies = p_oTree->GetSpecies();
    iType = p_oTree->GetType();

    //Get the Y coordinate
    p_oTree->GetValue(p_oPop->GetYCode(iSpecies, iType), &fY);

    //Do the count based on species, type, and Y
    if (iSpecies == 0 && iType == clTreePopulation::seedling && fY == 1)
      p_iNumTrees[0]++;
    else if (iSpecies == 1 && iType == clTreePopulation::seedling && fY == 1)
      p_iNumTrees[1]++;
    else if (iSpecies == 3 && iType == clTreePopulation::seedling && fY == 1)
      p_iNumTrees[2]++;
    else if (iSpecies == 0 && iType == clTreePopulation::sapling && fY == 1)
      p_iNumTrees[3]++;
    else if (iSpecies == 1 && iType == clTreePopulation::sapling && fY == 1)
      p_iNumTrees[4]++;
    else if (iSpecies == 3 && iType == clTreePopulation::sapling && fY == 1)
      p_iNumTrees[5]++;
    else if (iSpecies == 0 && iType == clTreePopulation::adult && fY == 1)
      p_iNumTrees[6]++;
    else if (iSpecies == 1 && iType == clTreePopulation::adult && fY == 1)
      p_iNumTrees[7]++;
    else if (iSpecies == 3 && iType == clTreePopulation::adult && fY == 1)
      p_iNumTrees[8]++;
    else if (iSpecies == 0 && iType == clTreePopulation::seedling && fY == 9)
      p_iNumTrees[9]++;
    else if (iSpecies == 1 && iType == clTreePopulation::seedling && fY == 9)
      p_iNumTrees[10]++;
    else if (iSpecies == 3 && iType == clTreePopulation::seedling && fY == 9)
      p_iNumTrees[11]++;
    else if (iSpecies == 0 && iType == clTreePopulation::sapling && fY == 9)
      p_iNumTrees[12]++;
    else if (iSpecies == 1 && iType == clTreePopulation::sapling && fY == 9)
      p_iNumTrees[13]++;
    else if (iSpecies == 3 && iType == clTreePopulation::sapling && fY == 9)
      p_iNumTrees[14]++;
    else if (iSpecies == 0 && iType == clTreePopulation::adult && fY == 9)
      p_iNumTrees[15]++;
    else if (iSpecies == 1 && iType == clTreePopulation::adult && fY == 9)
      p_iNumTrees[16]++;
    else if (iSpecies == 3 && iType == clTreePopulation::adult && fY == 9)
      p_iNumTrees[17]++;
    else if (iSpecies == 0 && iType == clTreePopulation::seedling && fY == 17)
      p_iNumTrees[18]++;
    else if (iSpecies == 1 && iType == clTreePopulation::seedling && fY == 17)
      p_iNumTrees[19]++;
    else if (iSpecies == 3 && iType == clTreePopulation::seedling && fY == 17)
      p_iNumTrees[20]++;
    else if (iSpecies == 0 && iType == clTreePopulation::sapling && fY == 17)
      p_iNumTrees[21]++;
    else if (iSpecies == 1 && iType == clTreePopulation::sapling && fY == 17)
      p_iNumTrees[22]++;
    else if (iSpecies == 3 && iType == clTreePopulation::sapling && fY == 17)
      p_iNumTrees[23]++;
    else if (iSpecies == 0 && iType == clTreePopulation::adult && fY == 17)
      p_iNumTrees[24]++;
    else if (iSpecies == 1 && iType == clTreePopulation::adult && fY == 17)
      p_iNumTrees[25]++;
    else if (iSpecies == 3 && iType == clTreePopulation::adult && fY == 17)
      p_iNumTrees[26]++;

    p_oTree = p_oAllTrees->NextTree();
  }

  //Compare to expected results
  //fExpected = 19.68532853;
  CPPUNIT_ASSERT(p_iNumTrees[0] < 40);
  fExpected = 501.4999955;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[1] - fExpected) / fExpected < 0.1);
  fExpected = 551.6154697;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[2] - fExpected) / fExpected < 0.1);
  fExpected = 995.9298623;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[3] - fExpected) / fExpected < 0.1);
  fExpected = 525.4779145;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[4] - fExpected) / fExpected < 0.1);
  fExpected = 581.6620475;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[5] - fExpected) / fExpected < 0.1);
  fExpected = 1000;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[6] - fExpected) / fExpected < 0.1);
  fExpected = 710.9495026;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[7] - fExpected) / fExpected < 0.1);
  fExpected = 681.1800199;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[8] - fExpected) / fExpected < 0.1);
  fExpected = 575.542209;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[9] - fExpected) / fExpected < 0.1);
  fExpected = 501.4999955;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[10] - fExpected) / fExpected < 0.1);
  fExpected = 551.6154697;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[11] - fExpected) / fExpected < 0.1);
  fExpected = 601.6871802;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[12] - fExpected) / fExpected < 0.1);
  fExpected = 525.4779145;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[13] - fExpected) / fExpected < 0.1);
  fExpected = 581.6620475;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[14] - fExpected) / fExpected < 0.1);
  fExpected = 704.1210772;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[15] - fExpected) / fExpected < 0.1);
  fExpected = 710.9495026;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[16] - fExpected) / fExpected < 0.1);
  fExpected = 681.1800199;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[17] - fExpected) / fExpected < 0.1);
  fExpected = 575.542209;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[18] - fExpected) / fExpected < 0.1);
  fExpected = 54.05000866;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[19] - fExpected) / fExpected < 0.1);
  fExpected = 627.1711494;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[20] - fExpected) / fExpected < 0.1);
  fExpected = 601.6871802;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[21] - fExpected) / fExpected < 0.1);
  fExpected = 70.51796365;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[22] - fExpected) / fExpected < 0.1);
  fExpected = 703.0365887;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[23] - fExpected) / fExpected < 0.1);
  fExpected = 704.1210772;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[24] - fExpected) / fExpected < 0.1);
  fExpected = 444.7268207;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[25] - fExpected) / fExpected < 0.1);
  fExpected = 887.0644161;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[26] - fExpected) / fExpected < 0.1);

  delete[] p_iNumTrees;
}

/////////////////////////////////////////////////////////////////////////////
//  NormalProcessingRun2()
/////////////////////////////////////////////////////////////////////////////
void clTestLogisticBiLevelMortality::NormalProcessingRun2(
    clSimManager *p_oSimManager) {
  clTreePopulation * p_oPop;
  clGridBase *p_oStormLight;
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  float fDiam, fY, fExpected;
  int *p_iNumTrees = new int[27], i, iSpecies, iType;

  for (i = 0; i < 27; i++) {
    p_iNumTrees[i] = 0;
  }

  p_oSimManager->ReadFile(WriteXMLFile2());
  p_oPop = (clTreePopulation *) p_oSimManager->GetPopulationObject(
      "treepopulation");
  p_oStormLight = p_oSimManager->GetGridObject("Storm Light");

  //Set our grid values
  fDiam = 15;
  p_oStormLight->SetValueOfCell(0, 0, p_oStormLight->GetFloatDataCode("Light"),
      fDiam);
  fDiam = 35;
  p_oStormLight->SetValueOfCell(0, 1, p_oStormLight->GetFloatDataCode("Light"),
      fDiam);
  fDiam = 80;
  p_oStormLight->SetValueOfCell(0, 2, p_oStormLight->GetFloatDataCode("Light"),
      fDiam);

  //Add 1000 trees to the plot of each kind
  for (i = 0; i < 1000; i++) {
    fDiam = 0.1;
    iSpecies = 0;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 0.2;
    iSpecies = 1;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 0.3;
    iSpecies = 3;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 2.5;
    iSpecies = 0;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 3.4;
    iSpecies = 1;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 5.4;
    iSpecies = 3;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 12.6;
    iSpecies = 0;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 30;
    iSpecies = 1;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 23.3;
    iSpecies = 3;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 0.1;
    iSpecies = 0;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 0.2;
    iSpecies = 1;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 0.3;
    iSpecies = 3;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 2.5;
    iSpecies = 0;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 3.4;
    iSpecies = 1;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 5.4;
    iSpecies = 3;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 12.6;
    iSpecies = 0;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 30;
    iSpecies = 1;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 23.3;
    iSpecies = 3;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 0.1;
    iSpecies = 0;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 0.2;
    iSpecies = 1;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 0.3;
    iSpecies = 3;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 2.5;
    iSpecies = 0;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 3.4;
    iSpecies = 1;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 5.4;
    iSpecies = 3;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 12.6;
    iSpecies = 0;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 30;
    iSpecies = 1;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 23.3;
    iSpecies = 3;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);
  }

  //Now run mortality, but skip storm light
  for (i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    clBehaviorBase *p_oBase = p_oSimManager->GetBehaviorObject(i);
    p_oBase->Action();
  }

  //Count up the trees
  p_oAllTrees = p_oPop->Find("all");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    iSpecies = p_oTree->GetSpecies();
    iType = p_oTree->GetType();

    //Get the Y coordinate
    p_oTree->GetValue(p_oPop->GetYCode(iSpecies, iType), &fY);

    //Do the count based on species, type, and Y
    if (iSpecies == 0 && iType == clTreePopulation::seedling && fY == 1)
      p_iNumTrees[0]++;
    else if (iSpecies == 1 && iType == clTreePopulation::seedling && fY == 1)
      p_iNumTrees[1]++;
    else if (iSpecies == 3 && iType == clTreePopulation::seedling && fY == 1)
      p_iNumTrees[2]++;
    else if (iSpecies == 0 && iType == clTreePopulation::sapling && fY == 1)
      p_iNumTrees[3]++;
    else if (iSpecies == 1 && iType == clTreePopulation::sapling && fY == 1)
      p_iNumTrees[4]++;
    else if (iSpecies == 3 && iType == clTreePopulation::sapling && fY == 1)
      p_iNumTrees[5]++;
    else if (iSpecies == 0 && iType == clTreePopulation::adult && fY == 1)
      p_iNumTrees[6]++;
    else if (iSpecies == 1 && iType == clTreePopulation::adult && fY == 1)
      p_iNumTrees[7]++;
    else if (iSpecies == 3 && iType == clTreePopulation::adult && fY == 1)
      p_iNumTrees[8]++;
    else if (iSpecies == 0 && iType == clTreePopulation::seedling && fY == 9)
      p_iNumTrees[9]++;
    else if (iSpecies == 1 && iType == clTreePopulation::seedling && fY == 9)
      p_iNumTrees[10]++;
    else if (iSpecies == 3 && iType == clTreePopulation::seedling && fY == 9)
      p_iNumTrees[11]++;
    else if (iSpecies == 0 && iType == clTreePopulation::sapling && fY == 9)
      p_iNumTrees[12]++;
    else if (iSpecies == 1 && iType == clTreePopulation::sapling && fY == 9)
      p_iNumTrees[13]++;
    else if (iSpecies == 3 && iType == clTreePopulation::sapling && fY == 9)
      p_iNumTrees[14]++;
    else if (iSpecies == 0 && iType == clTreePopulation::adult && fY == 9)
      p_iNumTrees[15]++;
    else if (iSpecies == 1 && iType == clTreePopulation::adult && fY == 9)
      p_iNumTrees[16]++;
    else if (iSpecies == 3 && iType == clTreePopulation::adult && fY == 9)
      p_iNumTrees[17]++;
    else if (iSpecies == 0 && iType == clTreePopulation::seedling && fY == 17)
      p_iNumTrees[18]++;
    else if (iSpecies == 1 && iType == clTreePopulation::seedling && fY == 17)
      p_iNumTrees[19]++;
    else if (iSpecies == 3 && iType == clTreePopulation::seedling && fY == 17)
      p_iNumTrees[20]++;
    else if (iSpecies == 0 && iType == clTreePopulation::sapling && fY == 17)
      p_iNumTrees[21]++;
    else if (iSpecies == 1 && iType == clTreePopulation::sapling && fY == 17)
      p_iNumTrees[22]++;
    else if (iSpecies == 3 && iType == clTreePopulation::sapling && fY == 17)
      p_iNumTrees[23]++;
    else if (iSpecies == 0 && iType == clTreePopulation::adult && fY == 17)
      p_iNumTrees[24]++;
    else if (iSpecies == 1 && iType == clTreePopulation::adult && fY == 17)
      p_iNumTrees[25]++;
    else if (iSpecies == 3 && iType == clTreePopulation::adult && fY == 17)
      p_iNumTrees[26]++;

    p_oTree = p_oAllTrees->NextTree();
  }

  //Compare to expected results
  //fExpected = 0.007628304;
  CPPUNIT_ASSERT(p_iNumTrees[0] < 10);
  fExpected = 126.128375;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[1] - fExpected) / fExpected < 0.2);
  fExpected = 167.8453491;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[2] - fExpected) / fExpected < 0.2);
  fExpected = 987.8392175;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[3] - fExpected) / fExpected < 0.1);
  fExpected = 145.0986604;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[4] - fExpected) / fExpected < 0.2);
  fExpected = 196.7941495;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[5] - fExpected) / fExpected < 0.2);
  fExpected = 1000;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[6] - fExpected) / fExpected < 0.1);
  fExpected = 359.348854;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[7] - fExpected) / fExpected < 0.1);
  fExpected = 316.0717659;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[8] - fExpected) / fExpected < 0.1);
  fExpected = 190.6476858;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[9] - fExpected) / fExpected < 0.1);
  fExpected = 126.128375;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[10] - fExpected) / fExpected < 0.2);
  fExpected = 167.8453491;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[11] - fExpected) / fExpected < 0.1);
  fExpected = 217.8272832;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[12] - fExpected) / fExpected < 0.1);
  fExpected = 145.0986604;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[13] - fExpected) / fExpected < 0.2);
  fExpected = 196.7941495;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[14] - fExpected) / fExpected < 0.1);
  fExpected = 349.0937184;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[15] - fExpected) / fExpected < 0.1);
  fExpected = 359.348854;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[16] - fExpected) / fExpected < 0.1);
  fExpected = 316.0717659;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[17] - fExpected) / fExpected < 0.1);
  fExpected = 190.6476858;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[18] - fExpected) / fExpected < 0.1);
  fExpected = 15.29804584;
  CPPUNIT_ASSERT(p_iNumTrees[19] < 30);
  fExpected = 246.6937894;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[20] - fExpected) / fExpected < 0.1);
  fExpected = 217.8272832;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[21] - fExpected) / fExpected < 0.1);
  //fExpected = 0.350670545;
  CPPUNIT_ASSERT(p_iNumTrees[22] < 10);
  fExpected = 347.4831772;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[23] - fExpected) / fExpected < 0.1);
  fExpected = 349.0937184;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[24] - fExpected) / fExpected < 0.1);
  fExpected = 87.95893562;
  CPPUNIT_ASSERT(p_iNumTrees[25] >= 60 && p_iNumTrees[25] < 110);
  fExpected = 698.0161558;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[26] - fExpected) / fExpected < 0.1);

  delete[] p_iNumTrees;
}

/////////////////////////////////////////////////////////////////////////////
//  NormalProcessingRun3()
/////////////////////////////////////////////////////////////////////////////
void clTestLogisticBiLevelMortality::NormalProcessingRun3(
    clSimManager *p_oSimManager) {
  clTreePopulation * p_oPop;
  clGridBase *p_oStormLight;
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  float fDiam, fY, fExpected;
  int *p_iNumTrees = new int[27], i, iSpecies, iType;

  for (i = 0; i < 27; i++) {
    p_iNumTrees[i] = 0;
  }

  p_oSimManager->ReadFile(WriteXMLFile3());
  p_oPop = (clTreePopulation *) p_oSimManager->GetPopulationObject(
      "treepopulation");
  p_oStormLight = p_oSimManager->GetGridObject("Storm Light");

  //Add 1000 trees to the plot of each kind
  for (i = 0; i < 1000; i++) {
    fDiam = 0.1;
    iSpecies = 0;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 0.2;
    iSpecies = 1;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 0.3;
    iSpecies = 3;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 2.5;
    iSpecies = 0;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 3.4;
    iSpecies = 1;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 5.4;
    iSpecies = 3;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 12.6;
    iSpecies = 0;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 30;
    iSpecies = 1;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 23.3;
    iSpecies = 3;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 1, iSpecies, iType, fDiam);

    fDiam = 0.1;
    iSpecies = 0;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 0.2;
    iSpecies = 1;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 0.3;
    iSpecies = 3;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 2.5;
    iSpecies = 0;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 3.4;
    iSpecies = 1;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 5.4;
    iSpecies = 3;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 12.6;
    iSpecies = 0;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 30;
    iSpecies = 1;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 23.3;
    iSpecies = 3;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 9, iSpecies, iType, fDiam);

    fDiam = 0.1;
    iSpecies = 0;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 0.2;
    iSpecies = 1;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 0.3;
    iSpecies = 3;
    iType = clTreePopulation::seedling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 2.5;
    iSpecies = 0;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 3.4;
    iSpecies = 1;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 5.4;
    iSpecies = 3;
    iType = clTreePopulation::sapling;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 12.6;
    iSpecies = 0;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 30;
    iSpecies = 1;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);

    fDiam = 23.3;
    iSpecies = 3;
    iType = clTreePopulation::adult;
    p_oTree = p_oPop->CreateTree(1, 17, iSpecies, iType, fDiam);
  }

  //Now run
  p_oSimManager->RunSim(1);

  //Count up the trees
  p_oAllTrees = p_oPop->Find("all");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    iSpecies = p_oTree->GetSpecies();
    iType = p_oTree->GetType();

    //Get the Y coordinate
    p_oTree->GetValue(p_oPop->GetYCode(iSpecies, iType), &fY);

    //Do the count based on species, type, and Y
    if (iSpecies == 0 && iType == clTreePopulation::seedling && fY == 1)
      p_iNumTrees[0]++;
    else if (iSpecies == 1 && iType == clTreePopulation::seedling && fY == 1)
      p_iNumTrees[1]++;
    else if (iSpecies == 3 && iType == clTreePopulation::seedling && fY == 1)
      p_iNumTrees[2]++;
    else if (iSpecies == 0 && iType == clTreePopulation::sapling && fY == 1)
      p_iNumTrees[3]++;
    else if (iSpecies == 1 && iType == clTreePopulation::sapling && fY == 1)
      p_iNumTrees[4]++;
    else if (iSpecies == 3 && iType == clTreePopulation::sapling && fY == 1)
      p_iNumTrees[5]++;
    else if (iSpecies == 0 && iType == clTreePopulation::adult && fY == 1)
      p_iNumTrees[6]++;
    else if (iSpecies == 1 && iType == clTreePopulation::adult && fY == 1)
      p_iNumTrees[7]++;
    else if (iSpecies == 3 && iType == clTreePopulation::adult && fY == 1)
      p_iNumTrees[8]++;
    else if (iSpecies == 0 && iType == clTreePopulation::seedling && fY == 9)
      p_iNumTrees[9]++;
    else if (iSpecies == 1 && iType == clTreePopulation::seedling && fY == 9)
      p_iNumTrees[10]++;
    else if (iSpecies == 3 && iType == clTreePopulation::seedling && fY == 9)
      p_iNumTrees[11]++;
    else if (iSpecies == 0 && iType == clTreePopulation::sapling && fY == 9)
      p_iNumTrees[12]++;
    else if (iSpecies == 1 && iType == clTreePopulation::sapling && fY == 9)
      p_iNumTrees[13]++;
    else if (iSpecies == 3 && iType == clTreePopulation::sapling && fY == 9)
      p_iNumTrees[14]++;
    else if (iSpecies == 0 && iType == clTreePopulation::adult && fY == 9)
      p_iNumTrees[15]++;
    else if (iSpecies == 1 && iType == clTreePopulation::adult && fY == 9)
      p_iNumTrees[16]++;
    else if (iSpecies == 3 && iType == clTreePopulation::adult && fY == 9)
      p_iNumTrees[17]++;
    else if (iSpecies == 0 && iType == clTreePopulation::seedling && fY == 17)
      p_iNumTrees[18]++;
    else if (iSpecies == 1 && iType == clTreePopulation::seedling && fY == 17)
      p_iNumTrees[19]++;
    else if (iSpecies == 3 && iType == clTreePopulation::seedling && fY == 17)
      p_iNumTrees[20]++;
    else if (iSpecies == 0 && iType == clTreePopulation::sapling && fY == 17)
      p_iNumTrees[21]++;
    else if (iSpecies == 1 && iType == clTreePopulation::sapling && fY == 17)
      p_iNumTrees[22]++;
    else if (iSpecies == 3 && iType == clTreePopulation::sapling && fY == 17)
      p_iNumTrees[23]++;
    else if (iSpecies == 0 && iType == clTreePopulation::adult && fY == 17)
      p_iNumTrees[24]++;
    else if (iSpecies == 1 && iType == clTreePopulation::adult && fY == 17)
      p_iNumTrees[25]++;
    else if (iSpecies == 3 && iType == clTreePopulation::adult && fY == 17)
      p_iNumTrees[26]++;

    p_oTree = p_oAllTrees->NextTree();
  }

  //Compare to expected results
  //fExpected = 19.68532853;
  CPPUNIT_ASSERT(p_iNumTrees[0] < 40);
  fExpected = 501.4999955;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[1] - fExpected) / fExpected < 0.1);
  fExpected = 551.6154697;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[2] - fExpected) / fExpected < 0.1);
  fExpected = 995.9298623;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[3] - fExpected) / fExpected < 0.1);
  fExpected = 525.4779145;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[4] - fExpected) / fExpected < 0.1);
  fExpected = 581.6620475;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[5] - fExpected) / fExpected < 0.1);
  fExpected = 1000;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[6] - fExpected) / fExpected < 0.1);
  fExpected = 710.9495026;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[7] - fExpected) / fExpected < 0.1);
  fExpected = 681.1800199;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[8] - fExpected) / fExpected < 0.1);
  //fExpected = 19.68532853;
  CPPUNIT_ASSERT(p_iNumTrees[9] < 40);
  fExpected = 501.4999955;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[10] - fExpected) / fExpected < 0.1);
  fExpected = 551.6154697;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[11] - fExpected) / fExpected < 0.1);
  fExpected = 995.9298623;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[12] - fExpected) / fExpected < 0.1);
  fExpected = 525.4779145;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[13] - fExpected) / fExpected < 0.1);
  fExpected = 581.6620475;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[14] - fExpected) / fExpected < 0.1);
  fExpected = 1000;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[15] - fExpected) / fExpected < 0.1);
  fExpected = 710.9495026;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[16] - fExpected) / fExpected < 0.1);
  fExpected = 681.1800199;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[17] - fExpected) / fExpected < 0.1);
  //fExpected = 19.68532853;
  CPPUNIT_ASSERT(p_iNumTrees[18] < 40);
  fExpected = 501.4999955;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[19] - fExpected) / fExpected < 0.1);
  fExpected = 551.6154697;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[20] - fExpected) / fExpected < 0.1);
  fExpected = 995.9298623;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[21] - fExpected) / fExpected < 0.1);
  fExpected = 525.4779145;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[22] - fExpected) / fExpected < 0.1);
  fExpected = 581.6620475;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[23] - fExpected) / fExpected < 0.1);
  fExpected = 1000;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[24] - fExpected) / fExpected < 0.1);
  fExpected = 710.9495026;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[25] - fExpected) / fExpected < 0.1);
  fExpected = 681.1800199;
  CPPUNIT_ASSERT(fabs(p_iNumTrees[26] - fExpected) / fExpected < 0.1);

  delete[] p_iNumTrees;
}

////////////////////////////////////////////////////////////////////////////
// ErrorProcessing()
////////////////////////////////////////////////////////////////////////////
void clTestLogisticBiLevelMortality::ErrorProcessing(
    clSimManager * p_oSimManager) {
  try {
    p_oSimManager->ReadFile(WriteXMLErrorFile1());
    CppUnit::Asserter::fail(
        "TestLogisticBiLevelMortality error processing didn't catch error for WriteXMLErrorFile1.");
  } catch (modelErr err) {
    if (strstr(err.cFunction, "clLogisticBiLevelMortality::DoShellSetup")
        == NULL) {
      CppUnit::Asserter::fail(
          "TestLogisticBiLevelMortality error processing caught wrong error for WriteXMLErrorFile1.");
    };
  }
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
////////////////////////////////////////////////////////////////////////////
char* clTestLogisticBiLevelMortality::WriteXMLFile1() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
      << "<paramFile fileCode=\"06010101\">" << "<plot>"
      << "<timesteps>1</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);

  oOut << "<behaviorList>" << "<behavior>"
      << "<behaviorName>Storm Light</behaviorName>" << "<version>1</version>"
      << "</behavior>" << "<behavior>"
      << "<behaviorName>Logistic Bi-Level Mortality</behaviorName>"
      << "<version>1</version>"
      << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_4\" type=\"Seedling\"/>" << "</behavior>"
      << "<behavior>" << "<behaviorName>removedead</behaviorName>"
      << "<version>1</version>"
      << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_4\" type=\"Seedling\"/>" << "</behavior>"
      << "</behaviorList>" << "<lightOther>"
      << "<li_stormLightRadius>15</li_stormLightRadius>"
      << "<li_stormLightSlope>-4</li_stormLightSlope>"
      << "<li_stormLightIntercept>95</li_stormLightIntercept>"
      << "<li_stormLightMaxDmgTime>3</li_stormLightMaxDmgTime>"
      << "<li_stormLightStoch>0</li_stormLightStoch>"
      << "<li_stormLightSnagMaxDmgTime>2</li_stormLightSnagMaxDmgTime>"
      << "<li_stormLightMinFullCanopy>9</li_stormLightMinFullCanopy>"
      << "</lightOther>";

  WriteMortParameters(oOut);
  oOut << "</paramFile>";

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
////////////////////////////////////////////////////////////////////////////
char* clTestLogisticBiLevelMortality::WriteXMLFile2() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
      << "<paramFile fileCode=\"06010101\">" << "<plot>"
      << "<timesteps>1</timesteps>" << "<yearsPerTimestep>3</yearsPerTimestep>";

  WriteCommonStuff(oOut);

  oOut << "<behaviorList>" << "<behavior>"
      << "<behaviorName>Storm Light</behaviorName>" << "<version>1</version>"
      << "</behavior>" << "<behavior>"
      << "<behaviorName>Logistic Bi-Level Mortality</behaviorName>"
      << "<version>1</version>"
      << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_4\" type=\"Seedling\"/>" << "</behavior>"
      << "<behavior>" << "<behaviorName>removedead</behaviorName>"
      << "<version>1</version>"
      << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_4\" type=\"Seedling\"/>" << "</behavior>"
      << "</behaviorList>" << "<lightOther>"
      << "<li_stormLightRadius>15</li_stormLightRadius>"
      << "<li_stormLightSlope>-4</li_stormLightSlope>"
      << "<li_stormLightIntercept>95</li_stormLightIntercept>"
      << "<li_stormLightMaxDmgTime>3</li_stormLightMaxDmgTime>"
      << "<li_stormLightStoch>0</li_stormLightStoch>"
      << "<li_stormLightSnagMaxDmgTime>2</li_stormLightSnagMaxDmgTime>"
      << "<li_stormLightMinFullCanopy>9</li_stormLightMinFullCanopy>"
      << "</lightOther>";

  WriteMortParameters(oOut);
  oOut << "</paramFile>";

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile3()
////////////////////////////////////////////////////////////////////////////
char* clTestLogisticBiLevelMortality::WriteXMLFile3() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
      << "<paramFile fileCode=\"06010101\">" << "<plot>"
      << "<timesteps>1</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);

  oOut << "<behaviorList>" << "<behavior>"
      << "<behaviorName>Logistic Bi-Level Mortality</behaviorName>"
      << "<version>1</version>"
      << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_4\" type=\"Seedling\"/>" << "</behavior>"
      << "<behavior>" << "<behaviorName>removedead</behaviorName>"
      << "<version>1</version>"
      << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_4\" type=\"Seedling\"/>" << "</behavior>"
      << "</behaviorList>";

  oOut << "<mortality>" << "<mo_logBilevLoLiteA>"
      << "<mo_lbllaVal species=\"Species_1\">-4.3</mo_lbllaVal>"
      << "<mo_lbllaVal species=\"Species_2\">0</mo_lbllaVal>"
      << "<mo_lbllaVal species=\"Species_4\">0.2</mo_lbllaVal>"
      << "</mo_logBilevLoLiteA>" << "<mo_logBilevLoLiteB>"
      << "<mo_lbllbVal species=\"Species_1\">3.92</mo_lbllbVal>"
      << "<mo_lbllbVal species=\"Species_2\">0.03</mo_lbllbVal>"
      << "<mo_lbllbVal species=\"Species_4\">0.024</mo_lbllbVal>"
      << "</mo_logBilevLoLiteB>" << "<mo_logBilevHiLiteThreshold>"
      << "<mo_lbhltVal species=\"Species_1\">20</mo_lbhltVal>"
      << "<mo_lbhltVal species=\"Species_2\">50</mo_lbhltVal>"
      << "<mo_lbhltVal species=\"Species_4\">70</mo_lbhltVal>"
      << "</mo_logBilevHiLiteThreshold>" << "</mortality>";
  oOut << "</paramFile>";

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile1()
////////////////////////////////////////////////////////////////////////////
char* clTestLogisticBiLevelMortality::WriteXMLErrorFile1() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
      << "<paramFile fileCode=\"06010101\">" << "<plot>"
      << "<timesteps>1</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);

  oOut << "<behaviorList>" << "<behavior>"
      << "<behaviorName>Storm Light</behaviorName>" << "<version>1</version>"
      << "</behavior>" << "<behavior>"
      << "<behaviorName>Logistic Bi-Level Mortality</behaviorName>"
      << "<version>1</version>"
      << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
      << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
      << "<applyTo species=\"Species_4\" type=\"Seedling\"/>" << "</behavior>"
      << "</behaviorList>" << "<lightOther>"
      << "<li_stormLightRadius>15</li_stormLightRadius>"
      << "<li_stormLightSlope>-4</li_stormLightSlope>"
      << "<li_stormLightIntercept>95</li_stormLightIntercept>"
      << "<li_stormLightMaxDmgTime>3</li_stormLightMaxDmgTime>"
      << "<li_stormLightSnagMaxDmgTime>2</li_stormLightSnagMaxDmgTime>"
      << "<li_stormLightStoch>0</li_stormLightStoch>"
      << "<li_stormLightMinFullCanopy>9</li_stormLightMinFullCanopy>"
      << "</lightOther>";

  oOut << "<mortality>" << "<mo_logBilevLoLiteA>"
      << "<mo_lbllaVal species=\"Species_1\">-4.3</mo_lbllaVal>"
      << "<mo_lbllaVal species=\"Species_2\">0</mo_lbllaVal>"
      << "<mo_lbllaVal species=\"Species_4\">0.2</mo_lbllaVal>"
      << "</mo_logBilevLoLiteA>" << "<mo_logBilevHiLiteA>"
      << "<mo_lbhlaVal species=\"Species_1\">0.3</mo_lbhlaVal>"
      << "<mo_lbhlaVal species=\"Species_2\">-2.88</mo_lbhlaVal>"
      << "<mo_lbhlaVal species=\"Species_4\">0.5</mo_lbhlaVal>"
      << "</mo_logBilevHiLiteA>" << "<mo_logBilevLoLiteB>"
      << "<mo_lbllbVal species=\"Species_1\">3.92</mo_lbllbVal>"
      << "<mo_lbllbVal species=\"Species_2\">0.03</mo_lbllbVal>"
      << "<mo_lbllbVal species=\"Species_4\">0.024</mo_lbllbVal>"
      << "</mo_logBilevLoLiteB>" << "<mo_logBilevHiLiteB>"
      << "<mo_lbhlbVal species=\"Species_1\">0.045</mo_lbhlbVal>"
      << "<mo_lbhlbVal species=\"Species_2\">8.86</mo_lbhlbVal>"
      << "<mo_lbhlbVal species=\"Species_4\">0.067</mo_lbhlbVal>"
      << "</mo_logBilevHiLiteB>" << "<mo_logBilevHiLiteThreshold>"
      << "<mo_lbhltVal species=\"Species_1\">120</mo_lbhltVal>"
      << "<mo_lbhltVal species=\"Species_2\">50</mo_lbhltVal>"
      << "<mo_lbhltVal species=\"Species_4\">70</mo_lbhltVal>"
      << "</mo_logBilevHiLiteThreshold>" << "</mortality>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
// WriteCommonStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestLogisticBiLevelMortality::WriteCommonStuff(std::fstream &oOut) {
  using namespace std;
  oOut << "<randomSeed>1</randomSeed>" << "<plot_lenX>100</plot_lenX>"
      << "<plot_lenY>100</plot_lenY>" << "<plot_latitude>55.37</plot_latitude>"
      << "</plot>" << "<trees>" << "<tr_speciesList>"
      << "<tr_species speciesName=\"Species_1\"/>"
      << "<tr_species speciesName=\"Species_2\"/>"
      << "<tr_species speciesName=\"Species_3\"/>"
      << "<tr_species speciesName=\"Species_4\"/>" << "</tr_speciesList>"
      << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>" << "<tr_minAdultDBH>"
      << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
      << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
      << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
      << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
      << "</tr_minAdultDBH>" << "<tr_maxSeedlingHeight>"
      << "<tr_mshVal species=\"Species_1\">2</tr_mshVal>"
      << "<tr_mshVal species=\"Species_2\">2</tr_mshVal>"
      << "<tr_mshVal species=\"Species_3\">2</tr_mshVal>"
      << "<tr_mshVal species=\"Species_4\">2</tr_mshVal>"
      << "</tr_maxSeedlingHeight>" << "</trees>" << "<allometry>"
      << "<tr_slopeOfAsymHeight>"
      << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
      << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
      << "<tr_soahVal species=\"Species_3\">0.0241</tr_soahVal>"
      << "<tr_soahVal species=\"Species_4\">0.0263</tr_soahVal>"
      << "</tr_slopeOfAsymHeight>" << "<tr_canopyHeight>"
      << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
      << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
      << "<tr_chVal species=\"Species_3\">40</tr_chVal>"
      << "<tr_chVal species=\"Species_4\">40</tr_chVal>"
      << "</tr_canopyHeight>" << "<tr_stdAsympCrownRad>"
      << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
      << "<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>"
      << "<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>"
      << "<tr_sacrVal species=\"Species_4\">0.0549</tr_sacrVal>"
      << "</tr_stdAsympCrownRad>" << "<tr_stdCrownRadExp>"
      << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
      << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
      << "<tr_screVal species=\"Species_3\">1.0</tr_screVal>"
      << "<tr_screVal species=\"Species_4\">1.0</tr_screVal>"
      << "</tr_stdCrownRadExp>" << "<tr_conversionDiam10ToDBH>"
      << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
      << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
      << "<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>"
      << "<tr_cdtdVal species=\"Species_4\">0.7059</tr_cdtdVal>"
      << "</tr_conversionDiam10ToDBH>" << "<tr_interceptDiam10ToDBH>"
      << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
      << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
      << "<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>"
      << "<tr_idtdVal species=\"Species_4\">0</tr_idtdVal>"
      << "</tr_interceptDiam10ToDBH>" << "<tr_stdAsympCrownHt>"
      << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
      << "<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>"
      << "<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>"
      << "<tr_sachVal species=\"Species_4\">0.389</tr_sachVal>"
      << "</tr_stdAsympCrownHt>" << "<tr_stdCrownHtExp>"
      << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
      << "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
      << "<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>"
      << "<tr_scheVal species=\"Species_4\">1.0</tr_scheVal>"
      << "</tr_stdCrownHtExp>" << "<tr_whatSeedlingHeightDiam>"
      << "<tr_wsehdVal species=\"Species_1\">1</tr_wsehdVal>"
      << "<tr_wsehdVal species=\"Species_2\">1</tr_wsehdVal>"
      << "<tr_wsehdVal species=\"Species_3\">1</tr_wsehdVal>"
      << "<tr_wsehdVal species=\"Species_4\">1</tr_wsehdVal>"
      << "</tr_whatSeedlingHeightDiam>" << "<tr_whatSaplingHeightDiam>"
      << "<tr_wsahdVal species=\"Species_1\">2</tr_wsahdVal>"
      << "<tr_wsahdVal species=\"Species_2\">2</tr_wsahdVal>"
      << "<tr_wsahdVal species=\"Species_3\">2</tr_wsahdVal>"
      << "<tr_wsahdVal species=\"Species_4\">2</tr_wsahdVal>"
      << "</tr_whatSaplingHeightDiam>" << "<tr_whatAdultHeightDiam>"
      << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
      << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
      << "<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>"
      << "<tr_wahdVal species=\"Species_4\">0</tr_wahdVal>"
      << "</tr_whatAdultHeightDiam>" << "<tr_seedlingLinearSlope>"
      << "<tr_selsVal species=\"Species_1\">0.9629</tr_selsVal>"
      << "<tr_selsVal species=\"Species_2\">0.8997</tr_selsVal>"
      << "<tr_selsVal species=\"Species_3\">0.8997</tr_selsVal>"
      << "<tr_selsVal species=\"Species_4\">3.857</tr_selsVal>"
      << "</tr_seedlingLinearSlope>" << "<tr_seedlingLinearIntercept>"
      << "<tr_seliVal species=\"Species_1\">0</tr_seliVal>"
      << "<tr_seliVal species=\"Species_2\">0</tr_seliVal>"
      << "<tr_seliVal species=\"Species_3\">0</tr_seliVal>"
      << "<tr_seliVal species=\"Species_4\">0</tr_seliVal>"
      << "</tr_seedlingLinearIntercept>" << "<tr_saplingReverseLinearSlope>"
      << "<tr_sarlsVal species=\"Species_1\">0.8</tr_sarlsVal>"
      << "<tr_sarlsVal species=\"Species_2\">0.7</tr_sarlsVal>"
      << "<tr_sarlsVal species=\"Species_3\">0.7</tr_sarlsVal>"
      << "<tr_sarlsVal species=\"Species_4\">0.6</tr_sarlsVal>"
      << "</tr_saplingReverseLinearSlope>"
      << "<tr_saplingReverseLinearIntercept>"
      << "<tr_sarliVal species=\"Species_1\">-0.758</tr_sarliVal>"
      << "<tr_sarliVal species=\"Species_2\">-0.33</tr_sarliVal>"
      << "<tr_sarliVal species=\"Species_3\">-0.33</tr_sarliVal>"
      << "<tr_sarliVal species=\"Species_4\">0.217</tr_sarliVal>"
      << "</tr_saplingReverseLinearIntercept>" << "<tr_whatAdultCrownRadDiam>"
      << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
      << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
      << "<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>"
      << "<tr_wacrdVal species=\"Species_4\">0</tr_wacrdVal>"
      << "</tr_whatAdultCrownRadDiam>" << "<tr_whatAdultCrownHeightHeight>"
      << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
      << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
      << "<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>"
      << "<tr_wachhVal species=\"Species_4\">0</tr_wachhVal>"
      << "</tr_whatAdultCrownHeightHeight>" << "<tr_whatSaplingCrownRadDiam>"
      << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
      << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
      << "<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>"
      << "<tr_wscrdVal species=\"Species_4\">0</tr_wscrdVal>"
      << "</tr_whatSaplingCrownRadDiam>" << "<tr_whatSaplingCrownHeightHeight>"
      << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
      << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
      << "<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>"
      << "<tr_wschhVal species=\"Species_4\">0</tr_wschhVal>"
      << "</tr_whatSaplingCrownHeightHeight>" << "</allometry>";
}

/////////////////////////////////////////////////////////////////////////////
// WriteMortParameters()
/////////////////////////////////////////////////////////////////////////////
void clTestLogisticBiLevelMortality::WriteMortParameters(std::fstream &oOut) {
  using namespace std;
  oOut << "<mortality>" << "<mo_logBilevLoLiteA>"
      << "<mo_lbllaVal species=\"Species_1\">-4.3</mo_lbllaVal>"
      << "<mo_lbllaVal species=\"Species_2\">0</mo_lbllaVal>"
      << "<mo_lbllaVal species=\"Species_4\">0.2</mo_lbllaVal>"
      << "</mo_logBilevLoLiteA>" << "<mo_logBilevHiLiteA>"
      << "<mo_lbhlaVal species=\"Species_1\">0.3</mo_lbhlaVal>"
      << "<mo_lbhlaVal species=\"Species_2\">-2.88</mo_lbhlaVal>"
      << "<mo_lbhlaVal species=\"Species_4\">0.5</mo_lbhlaVal>"
      << "</mo_logBilevHiLiteA>" << "<mo_logBilevLoLiteB>"
      << "<mo_lbllbVal species=\"Species_1\">3.92</mo_lbllbVal>"
      << "<mo_lbllbVal species=\"Species_2\">0.03</mo_lbllbVal>"
      << "<mo_lbllbVal species=\"Species_4\">0.024</mo_lbllbVal>"
      << "</mo_logBilevLoLiteB>" << "<mo_logBilevHiLiteB>"
      << "<mo_lbhlbVal species=\"Species_1\">0.045</mo_lbhlbVal>"
      << "<mo_lbhlbVal species=\"Species_2\">0.0886</mo_lbhlbVal>"
      << "<mo_lbhlbVal species=\"Species_4\">0.067</mo_lbhlbVal>"
      << "</mo_logBilevHiLiteB>" << "<mo_logBilevHiLiteThreshold>"
      << "<mo_lbhltVal species=\"Species_1\">20</mo_lbhltVal>"
      << "<mo_lbhltVal species=\"Species_2\">50</mo_lbhltVal>"
      << "<mo_lbhltVal species=\"Species_4\">70</mo_lbhltVal>"
      << "</mo_logBilevHiLiteThreshold>" << "</mortality>";
}
