//---------------------------------------------------------------------------
// TestMastingNonSpatialDisperse
//---------------------------------------------------------------------------
#if !defined(TestNonSpatialDisperse_H)
  #define TestNonSpatialDisperse_H

  #include <cppunit/TestFixture.h>
  #include <cppunit/TestCaller.h>

  class clGridBase;
  class clSimManager;

  /**
   * This class tests the masting non-spatial disperse behavior 
   * (clMastingNonSpatialDisperse).
   */
  class clTestMastingNonSpatialDisperse : public CppUnit::TestFixture
  {

  public:

  /**
   * Rolls together all the tests.
   */
  void TestMastingNonSpatialDisperse();

  private:

  /**
   * Tests masting non-spatial dispersal, run 1. It does the following:
   * Checks that the correct params are used for mast vs. nonmast; checks that
   * grouping happens correctly; checks that when there are no trees, basal 
   * area is split correctly. WriteXMLFile1 does setup.
   * @param p_oSimManager Sim manager to direct testing.
   */
  void NormalProcessingRun1(clSimManager *p_oSimManager);
  
  /**
   * Tests masting non-spatial dispersal, run 2. It does the following:
   * Checks that grouping happens correctly; checks that basal area splits 
   * happen correctly; and checks multi-year timestep. WriteXMLFile2 does setup.
   * @param p_oSimManager Sim manager to direct testing.
   */
  void NormalProcessingRun2(clSimManager *p_oSimManager);
  
  /**
   * Tests masting non-spatial dispersal, run 3. It does the following:
   * Checks using different draws appropriately. WriteXMLFile3 does setup.
   * @param p_oSimManager Sim manager to direct testing.
   */
  void NormalProcessingRun3(clSimManager *p_oSimManager);
  
  /**
   * Tests error processing. This uses WriteXMLErrorFileX to introduce errors 
   * and make sure that they are caught.
   * @param p_oSimManager Sim Manager object to call for runs.
   */
  void TestErrorProcessing(clSimManager *p_oSimManager);  

  /**
   * Writes a parameter file to direct testing for run 1.
   * @return Filename written.
   */
  char * WriteXMLFile1();

  /**
   * Writes a parameter file to direct testing for run 2.
   * @return Filename written.
   */
  char * WriteXMLFile2();
  
  /**
   * Writes a parameter file to direct testing for run 3.
   * @return Filename written.
   */
  char * WriteXMLFile3();
  
  /**
   * Writes an error file where masting inverse gaussian mu is not > 0.
   * @return Filename written.
   */
  char* WriteXMLErrorFile1();
  
  /**
   * Writes an error file where non-masting inverse gaussian mu is not > 0.
   * @return Filename written.
   */
  char* WriteXMLErrorFile2();

  /**
   * Writes an error file where masting inverse gaussian lambda is not > 0.
   * @return Filename written.
   */
  char* WriteXMLErrorFile3();

  /**
   * Writes an error file where non-masting inverse gaussian lambda is not > 0.
   * @return Filename written.
   */
  char* WriteXMLErrorFile4();
  
  /**
  * Writes tree stuff.
  * @param oOut File stream to write to.
  */
  void WriteTrees(std::fstream &oOut);
  
  /**
  * Writes allometry stuff.
  * @param oOut File stream to write to.
  */
  void WriteAllometry(std::fstream &oOut);
    
};
//---------------------------------------------------------------------------
#endif // TestMastingNonSpatialDispersal_H
