//---------------------------------------------------------------------------
// TestMichMenPhotoinhibition
//---------------------------------------------------------------------------
#if !defined(TestMichMenPhotoinhibition_H)
  #define TestMichMenPhotoinhibition_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the clMichMenPhotoinhibition class.
*/
class clTestMichMenPhotoinhibition : public CppUnit::TestFixture {

  public:

  /**
  * Combines the testing into one function.
  */
  void TestMichMenPhotoinhibition();

  protected:

  /**
  * Tests height growth when the timestep length is one year. WriteXMLFile1() is
  * used to set up the run. 10 trees are created of four species. GLIs are
  * assigned to them, and then growth is run. Their diameters, heights, and
  * "Growth" values are checked to make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test1Run1YearPerTimestep(clSimManager *p_oSimManager);

  /**
  * Tests height growth when the timestep length is three years. WriteXMLFile2()
  * is used to set up the run. 10 trees are created of four species. GLIs are
  * assigned to them, and then growth is run. Their diameters, heights, and
  * "Growth" values are checked to make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test1Run3YearsPerTimestep(clSimManager *p_oSimManager);

  /**
  * Tests error processing.
  * @param p_oSimManager Sim Manager to do runs.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to direct testing. Timestep length is 1
  * year. This contains four species. Relative height growth is applied to all
  * species. Diameter incrementer applied to all as well.
  *
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a parameter file to direct testing. Timestep length is 3
  * years. This contains four species. Relative height growth is applied to all
  * species. Diameter incrementer applied to all as well.
  *
  * @return Filename written.
  */
  char* WriteXMLFile2();

  /**
  * Writes an error file where the beta parameter = 0.
  * @return File name of the freshly written parameter file.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes common portions of the test parameter files.
  * @param oOut File stream to write to.
  */
  void WriteCommonStuff(std::fstream &oOut);

};
//---------------------------------------------------------------------------
#endif // TestMichMenPhotoinhibition_H
