//---------------------------------------------------------------------------
// TestOutput.cpp
//---------------------------------------------------------------------------
#include "TestOutput.h"
#include "SimManager.h"
#include "Output.h"
#include "PlatformFuncs.h"
#include <stdio.h>
#include <fstream.h>
#include <math.h>
#include <iostream>

/////////////////////////////////////////////////////////////////////////////
// TestOutput()
/////////////////////////////////////////////////////////////////////////////
void clTestOutput::TestOutput() {
  clSimManager *p_oSimManager = new clSimManager(3, 3, "");

  NormalProcessingRun1(p_oSimManager);
  
  delete p_oSimManager;
}

/////////////////////////////////////////////////////////////////////////////
// NormalProcessingRun1()
/////////////////////////////////////////////////////////////////////////////
void clTestOutput::NormalProcessingRun1(clSimManager *p_oSimManager) {

  //Feed our file to the sim manager
  p_oSimManager->ReadFile(WriteXMLFile1());
   
  //Verify that the output file has been created
  CPPUNIT_ASSERT(DoesFileExist("testoutput.gz.tar"));
  
  //Run one timestep
  p_oSimManager->RunSim(1);
  
  //Verify that the output file has been created
  CPPUNIT_ASSERT(DoesFileExist("testoutput.gz.tar"));
  
  DeleteThisFile("testoutput.gz.tar");
  CPPUNIT_ASSERT(!DoesFileExist("testoutput.gz.tar"));
  
  p_oSimManager->ReadFile(WriteXMLFile2());
   
  //Verify that the output file has been created
  CPPUNIT_ASSERT(DoesFileExist("testoutput1.gz.tar"));
  
  //Run one timestep
  p_oSimManager->RunSim(1);
  
  //Verify that the output file has been created
  CPPUNIT_ASSERT(DoesFileExist("testoutput1.gz.tar"));
  
  DeleteThisFile("testoutput1.gz.tar");
  CPPUNIT_ASSERT(!DoesFileExist("testoutput1.gz.tar"));
}


/////////////////////////////////////////////////////////////////////////////
// WriteXMLFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestOutput::WriteXMLFile1()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06080101\">"
       << "<plot>"
       << "<timesteps>400</timesteps>"
       << "<rt_timestep>0</rt_timestep>"
       << "<randomSeed>0</randomSeed>"
       << "<yearsPerTimestep>5.0</yearsPerTimestep>"
       << "<plot_lenX>500.0</plot_lenX>"
       << "<plot_lenY>500.0</plot_lenY>"
       << "<plot_latitude>41.92</plot_latitude>"
       << "<plot_title>Default parameter file-use for testing only</plot_title>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"ACRU\"/>"
       << "<tr_species speciesName=\"ACSA\"/>"
       << "<tr_species speciesName=\"BEAL\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"s1.0\"/>"
       << "<tr_sizeClass sizeKey=\"s10.0\"/>"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"ACRU\">"
       << "<tr_initialDensity sizeClass=\"s1.0\">25.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"ACSA\">"
       << "<tr_initialDensity sizeClass=\"s1.0\">25.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"BEAL\">"
       << "<tr_initialDensity sizeClass=\"s1.0\">25.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>"
       << "<tr_seedDiam10Cm>0.2</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"ACRU\">10.0</tr_madVal>"
       << "<tr_madVal species=\"ACSA\">10.0</tr_madVal>"
       << "<tr_madVal species=\"BEAL\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"ACRU\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"ACSA\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"BEAL\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>quadratlight</behaviorName>"
       << "<version>1.0</version>"
       << "<applyTo species=\"ACRU\" type=\"Seedling\"/>"
       << "<applyTo species=\"ACSA\" type=\"Seedling\"/>"
       << "<applyTo species=\"BEAL\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>output</behaviorName>"
       << "<version>1.0</version>"
       << "<applyTo species=\"ACRU\" type=\"Seedling\"/>"
       << "<applyTo species=\"ACRU\" type=\"Sapling\"/>"
       << "<applyTo species=\"ACRU\" type=\"Adult\"/>"
       << "<applyTo species=\"ACSA\" type=\"Seedling\"/>"
       << "<applyTo species=\"ACSA\" type=\"Sapling\"/>"
       << "<applyTo species=\"ACSA\" type=\"Adult\"/>"
       << "<applyTo species=\"BEAL\" type=\"Seedling\"/>"
       << "<applyTo species=\"BEAL\" type=\"Sapling\"/>"
       << "<applyTo species=\"BEAL\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<allometry>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"ACRU\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"ACSA\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"BEAL\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"ACRU\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"ACSA\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"BEAL\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"ACRU\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"ACSA\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"BEAL\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"ACRU\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"ACSA\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"BEAL\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"ACRU\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"ACSA\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"BEAL\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"ACRU\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"ACSA\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"BEAL\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"ACRU\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"ACSA\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"BEAL\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"ACRU\">30.0</tr_chVal>"
       << "<tr_chVal species=\"ACSA\">30.0</tr_chVal>"
       << "<tr_chVal species=\"BEAL\">30.0</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"ACRU\">0.108</tr_sacrVal>"
       << "<tr_sacrVal species=\"ACSA\">0.107</tr_sacrVal>"
       << "<tr_sacrVal species=\"BEAL\">0.109</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"ACRU\">1.0</tr_screVal>"
       << "<tr_screVal species=\"ACSA\">1.0</tr_screVal>"
       << "<tr_screVal species=\"BEAL\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"ACRU\">0.49</tr_sachVal>"
       << "<tr_sachVal species=\"ACSA\">0.58</tr_sachVal>"
       << "<tr_sachVal species=\"BEAL\">0.54</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"ACRU\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"ACSA\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"BEAL\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"ACRU\">0.75</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"ACSA\">0.75</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"BEAL\">0.75</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"ACRU\">0.0</tr_idtdVal>"
       << "<tr_idtdVal species=\"ACSA\">0.0</tr_idtdVal>"
       << "<tr_idtdVal species=\"BEAL\">0.0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"ACRU\">0.063</tr_soahVal>"
       << "<tr_soahVal species=\"ACSA\">0.062333334</tr_soahVal>"
       << "<tr_soahVal species=\"BEAL\">0.063</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"ACRU\">0.03</tr_sohdVal>"
       << "<tr_sohdVal species=\"ACSA\">0.03</tr_sohdVal>"
       << "<tr_sohdVal species=\"BEAL\">0.03</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "</allometry>"
       << "<gliLight>"
       << "<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>120</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>270</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.779</li_minSunAngle>"
       << "<li_numAltGrids>12</li_numAltGrids>"
       << "<li_numAziGrids>18</li_numAziGrids>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"ACRU\">0.399</li_lecVal>"
       << "<li_lecVal species=\"ACSA\">0.399</li_lecVal>"
       << "<li_lecVal species=\"BEAL\">0.399</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "<li_snagAgeClass1>0</li_snagAgeClass1>"
       << "<li_snagAgeClass2>0</li_snagAgeClass2>"
       << "</gliLight>"
       << "<quadratGliLight>"
       << "<li_minSunAngle>0.779</li_minSunAngle>"
       << "<li_numAltGrids>12</li_numAltGrids>"
       << "<li_numAziGrids>18</li_numAziGrids>"
       << "<li_quadratLightHeight>0.675</li_quadratLightHeight>"
       << "</quadratGliLight>"
       << "<output>"
       << "<ou_filename>testoutput.gz.tar</ou_filename>"
       << "<ou_treeInfo species=\"ACRU\" type=\"Seedling\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>Diam10</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"ACRU\" type=\"Sapling\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>Diam10</ou_float>"
       << "<ou_float>DBH</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"ACRU\" type=\"Adult\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>DBH</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"ACSA\" type=\"Seedling\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>Diam10</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"ACSA\" type=\"Sapling\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>Diam10</ou_float>"
       << "<ou_float>DBH</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"ACSA\" type=\"Adult\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>DBH</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"BEAL\" type=\"Seedling\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>Diam10</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"BEAL\" type=\"Sapling\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>Diam10</ou_float>"
       << "<ou_float>DBH</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"BEAL\" type=\"Adult\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>DBH</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_gridInfo gridName=\"Quadrat GLI\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>GLI</ou_float>"
       << "</ou_gridInfo>"
       << "</output>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
// WriteXMLFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestOutput::WriteXMLFile2()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06080101\">"
       << "<plot>"
       << "<timesteps>400</timesteps>"
       << "<rt_timestep>0</rt_timestep>"
       << "<randomSeed>0</randomSeed>"
       << "<yearsPerTimestep>5.0</yearsPerTimestep>"
       << "<plot_lenX>500.0</plot_lenX>"
       << "<plot_lenY>500.0</plot_lenY>"
       << "<plot_latitude>41.92</plot_latitude>"
       << "<plot_title>Default parameter file-use for testing only</plot_title>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"ACRU\"/>"
       << "<tr_species speciesName=\"ACSA\"/>"
       << "<tr_species speciesName=\"BEAL\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"s1.0\"/>"
       << "<tr_sizeClass sizeKey=\"s10.0\"/>"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"ACRU\">"
       << "<tr_initialDensity sizeClass=\"s1.0\">25.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"ACSA\">"
       << "<tr_initialDensity sizeClass=\"s1.0\">25.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"BEAL\">"
       << "<tr_initialDensity sizeClass=\"s1.0\">25.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>"
       << "<tr_seedDiam10Cm>0.2</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"ACRU\">10.0</tr_madVal>"
       << "<tr_madVal species=\"ACSA\">10.0</tr_madVal>"
       << "<tr_madVal species=\"BEAL\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"ACRU\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"ACSA\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"BEAL\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>quadratlight</behaviorName>"
       << "<version>1.0</version>"
       << "<applyTo species=\"ACRU\" type=\"Seedling\"/>"
       << "<applyTo species=\"ACSA\" type=\"Seedling\"/>"
       << "<applyTo species=\"BEAL\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>output</behaviorName>"
       << "<version>1.0</version>"
       << "<applyTo species=\"ACRU\" type=\"Seedling\"/>"
       << "<applyTo species=\"ACRU\" type=\"Sapling\"/>"
       << "<applyTo species=\"ACRU\" type=\"Adult\"/>"
       << "<applyTo species=\"ACSA\" type=\"Seedling\"/>"
       << "<applyTo species=\"ACSA\" type=\"Sapling\"/>"
       << "<applyTo species=\"ACSA\" type=\"Adult\"/>"
       << "<applyTo species=\"BEAL\" type=\"Seedling\"/>"
       << "<applyTo species=\"BEAL\" type=\"Sapling\"/>"
       << "<applyTo species=\"BEAL\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<allometry>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"ACRU\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"ACSA\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"BEAL\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"ACRU\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"ACSA\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"BEAL\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"ACRU\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"ACSA\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"BEAL\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"ACRU\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"ACSA\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"BEAL\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"ACRU\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"ACSA\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"BEAL\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"ACRU\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"ACSA\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"BEAL\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"ACRU\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"ACSA\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"BEAL\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"ACRU\">30.0</tr_chVal>"
       << "<tr_chVal species=\"ACSA\">30.0</tr_chVal>"
       << "<tr_chVal species=\"BEAL\">30.0</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"ACRU\">0.108</tr_sacrVal>"
       << "<tr_sacrVal species=\"ACSA\">0.107</tr_sacrVal>"
       << "<tr_sacrVal species=\"BEAL\">0.109</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"ACRU\">1.0</tr_screVal>"
       << "<tr_screVal species=\"ACSA\">1.0</tr_screVal>"
       << "<tr_screVal species=\"BEAL\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"ACRU\">0.49</tr_sachVal>"
       << "<tr_sachVal species=\"ACSA\">0.58</tr_sachVal>"
       << "<tr_sachVal species=\"BEAL\">0.54</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"ACRU\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"ACSA\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"BEAL\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"ACRU\">0.75</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"ACSA\">0.75</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"BEAL\">0.75</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"ACRU\">0.0</tr_idtdVal>"
       << "<tr_idtdVal species=\"ACSA\">0.0</tr_idtdVal>"
       << "<tr_idtdVal species=\"BEAL\">0.0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"ACRU\">0.063</tr_soahVal>"
       << "<tr_soahVal species=\"ACSA\">0.062333334</tr_soahVal>"
       << "<tr_soahVal species=\"BEAL\">0.063</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"ACRU\">0.03</tr_sohdVal>"
       << "<tr_sohdVal species=\"ACSA\">0.03</tr_sohdVal>"
       << "<tr_sohdVal species=\"BEAL\">0.03</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "</allometry>"
       << "<gliLight>"
       << "<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>120</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>270</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.779</li_minSunAngle>"
       << "<li_numAltGrids>12</li_numAltGrids>"
       << "<li_numAziGrids>18</li_numAziGrids>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"ACRU\">0.399</li_lecVal>"
       << "<li_lecVal species=\"ACSA\">0.399</li_lecVal>"
       << "<li_lecVal species=\"BEAL\">0.399</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "<li_snagAgeClass1>0</li_snagAgeClass1>"
       << "<li_snagAgeClass2>0</li_snagAgeClass2>"
       << "</gliLight>"
       << "<quadratGliLight>"
       << "<li_minSunAngle>0.779</li_minSunAngle>"
       << "<li_numAltGrids>12</li_numAltGrids>"
       << "<li_numAziGrids>18</li_numAziGrids>"
       << "<li_quadratLightHeight>0.675</li_quadratLightHeight>"
       << "</quadratGliLight>"
       << "<output>"
       << "<ou_filename>testoutput1</ou_filename>"
       << "<ou_treeInfo species=\"ACRU\" type=\"Seedling\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>Diam10</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"ACRU\" type=\"Sapling\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>Diam10</ou_float>"
       << "<ou_float>DBH</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"ACRU\" type=\"Adult\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>DBH</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"ACSA\" type=\"Seedling\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>Diam10</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"ACSA\" type=\"Sapling\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>Diam10</ou_float>"
       << "<ou_float>DBH</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"ACSA\" type=\"Adult\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>DBH</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"BEAL\" type=\"Seedling\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>Diam10</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"BEAL\" type=\"Sapling\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>Diam10</ou_float>"
       << "<ou_float>DBH</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_treeInfo species=\"BEAL\" type=\"Adult\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>X</ou_float>"
       << "<ou_float>Y</ou_float>"
       << "<ou_float>DBH</ou_float>"
       << "<ou_float>Height</ou_float>"
       << "</ou_treeInfo>"
       << "<ou_gridInfo gridName=\"Quadrat GLI\">"
       << "<ou_saveFreq>1</ou_saveFreq>"
       << "<ou_float>GLI</ou_float>"
       << "</ou_gridInfo>"
       << "</output>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

