//---------------------------------------------------------------------------
// TestPostHarvestSkiddingMort
//---------------------------------------------------------------------------
#if !defined(TestPostHarvestSkiddingMort_H)
  #define TestPostHarvestSkiddingMort_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>


class clSimManager;
class clTreePopulation;
class clTree;

/**
* Tests the PostHarvestSkiddingMort class descended from clMortalityBase.
* @author Mark Vanderwel
*/
class clTestPostHarvestSkiddingMort : public CppUnit::TestFixture
{
  public:

  /**
  * Tests the post-harvest skidding mortality class.
  * <ul>
  * <li>2000 trees are created in each of 5 different grid cells.
  * <li>Cells 1-3 were cut this timestep, cell 4 was not cut, and cell 5 was cut 6 years earlier.
  * <li>Cell 2 has higher harvest intensity than cell 1.
  * <li>Cell 3 has larger trees than cell 1.
  * <li>Simulation is run for 1 2-y timestep.
  * <li>A second run is performed where the HarvInten code is not registered. 
  * It is otherwise the same as run 1, cell 1. The behavior should treat HarvInten as having a value of 0.
  * <li>The number of trees of each species remaining in each cell is compared to 
  * expected values (+/- 5%).
  * </ul>
  */


  void TestPostHarvestSkiddingMort();

  protected:

  /**
  * Writes a parameter file with the specifications given for
  * TestPostHarvestSkiddingMort(), runs 1 and 2. If parameter value of 1 is passed, 
  * then the Harvest Interface behavior is included in the parameter file.
  * @return Filename written.
  */
  char * WriteXMLFile1(int iRun);
  
  
};
//---------------------------------------------------------------------------
#endif // TestPostHarvestSkiddingMort_H
