//---------------------------------------------------------------------------
// TestPowerHeightGrowth
//---------------------------------------------------------------------------
#if !defined(TestPowerHeightGrowth_H)
  #define TestPowerHeightGrowth_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the clPowerHeightGrowth class.
*/
class clTestPowerHeightGrowth : public CppUnit::TestFixture {

  public:

  /**
  * Combines the testing into one function.
  */
  void TestPowerHeightGrowth();

  protected:

  /**
  * Tests height growth when the timestep length is one year. WriteXMLFile1() is
  * used to set up the run. 10 trees are created of four species. Growth is
  * run. Their diameters, heights, and "Growth" values are checked to make
  * sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void TestRun1YearPerTimestep(clSimManager *p_oSimManager);

  /**
  * Tests height growth when the timestep length is three years. WriteXMLFile2()
  * is used to set up the run. 10 trees are created of four species. Growth is
  * run. Their diameters, heights, and "Growth" values are checked to make sure
  * they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void TestRun3YearsPerTimestep(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to direct testing for run 1. Timestep length is 1
  * year. This contains four species. Power height growth is applied to 3
  * species. Diameter incrementer applied as well.
  *
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a parameter file to direct testing for run 1. Timestep length is 3
  * years. This contains four species. Relative height growth is applied to 3
  * species. Diameter incrementer applied as well.
  *
  * @return Filename written.
  */
  char* WriteXMLFile2();

  /**
  * Writes common portions of the test parameter files.
  * @param oOut File stream to write to.
  */
  void WriteCommonStuff(std::fstream &oOut);

};
//---------------------------------------------------------------------------
#endif // TestPowerHeightGrowth_H
