//---------------------------------------------------------------------------
// TestRandomBrowse
//---------------------------------------------------------------------------
#if !defined(TestRandomBrowse_H)
  #define TestRandomBrowse_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clSimManager;

/**
* Tests the clRandomBrowse class.
* @author Lora E. Murphy
*/
class clTestRandomBrowse : public CppUnit::TestFixture
{
  public:

  /**
  * Consolidates the testing into a single function.
  */
  void TestRandomBrowse();
  
  protected:
  
  /**
  * Tests normal processing run 1.
  * First run: WriteXMLFile1() creates the parameter file.  The file has the
  * following characteristics:
  * <ul>
  * <li>Timestep length of 1 year.</li>
  * <li>5 species: Species 1, Species 2, Species 3, Species 4, Species 5.</li>
  * <li>Random browse behavior applied to seedlings and saplings of Species 
  * 1, 4 and 5, and seedlings of species 3.</li>
  * </ul>
  * The function does the following:
  * <ol>
  * <li>1000 trees are created of each tree species/type combination for 
  * seedlings, saplings, and adults.</li>
  * <li> The model is run for 1 timestep.</li>
  * <li> The number browsed are counted.</li>
  * <li> The counts are compared to expected results, give or take 5%.</li>
  * <li> The model is run for another timestep. </li>
  * <li> The number browsed is expected to be roughly the same, but not exactly
  * the same (i.e. a new set of browsed individuals has been chosen).</li>
  * </ol>
  * @param p_oSimManager Sim manager.
  */
  void TestNormalRun1(clSimManager *p_oSimManager);
  
  /**
  * Tests normal processing run 2.
  * WriteXMLFile2() creates the parameter file.  The run is exactly the same
  * as run 1except the timestep length is three years, so the browse 
  * probabilities are compounded appropriately.
  * @param p_oSimManager Sim manager.
  */
  void TestNormalRun2(clSimManager *p_oSimManager);
  
  /**
  * Tests normal processing run 3.
  * WriteXMLFile3() creates the parameter file. The probabilities are done on
  * a normal random draw. This creates a large number of probabilities to
  * test the draw.
  * @param p_oSimManager Sim manager.
  */
  void TestNormalRun3(clSimManager *p_oSimManager);
  
  /**
  * Tests error processing.
  * @param p_oSimManager Sim manager.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file with the specifications given for
  * TestRandomBrowse(), run 1.
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file with the specifications given for
  * TestRandomBrowse(), run 2.
  * @return Filename written.
  */
  char * WriteXMLFile2();
  
  /**
  * Writes a parameter file with the specifications given for
  * TestRandomBrowse(), run 3.
  * @return Filename written.
  */
  char * WriteXMLFile3();
  
  /**
  * Writes a parameter file where the browse probabilities are not all between
  * 0 and 1.
  * @return Filename written.
  */
  char * WriteXMLErrorFile1();

};
//---------------------------------------------------------------------------
#endif // TestRandomBrowse_H
