//---------------------------------------------------------------------------
// TestRelativeGrowth
//---------------------------------------------------------------------------
#if !defined(TestRelativeGrowth_H)
  #define TestRelativeGrowth_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the clRelativeGrowth class.  This tests all six types of relative
* growth - non-limited relative growth, relative growth limited to constant
* radial increment, relative growth limited to constant basal area increment,
* non-limited relative growth - diameter update only, relative growth limited to
* constant radial increment - diameter update only, and relative growth limited
* to constant basal area increment - diameter update only.
*/
class clTestRelativeGrowth : public CppUnit::TestFixture {

  public:

  /**
  * Combines the testing into one function.
  */
  void TestRelativeGrowth();

  protected:

  /**
  * Tests the growth when the timestep length is one year. WriteXMLFile1()
  * is used to set up the run. 10 trees are created of four species, assigned 
  * to all six growth types and with growth levels both above and below limits.
  * GLIs are assigned to them, and then growth is run. Their diameters, heights,
  * and "Growth" values are checked to make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test1Run1YearPerTimestep(clSimManager *p_oSimManager);

  /**
  * Tests the growth when the timestep length is three years. WriteXMLFile2()
  * is used to set up the run. 10 trees are created of three species, assigned
  * to all six growth types and with growth levels both above and below limits.
  * GLIs are assigned to them, and then growth is run. Their diameters, heights,
  * and "Growth" values are checked to make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test1Run3YearsPerTimestep(clSimManager *p_oSimManager);

  /**
  * Tests the growth when the timestep length is one year. WriteXMLFile3() is 
  * used to set up the run. 10 trees are created of four species, assigned to
  * all six growth types and with growth levels both above and below limits.
  * There are non-1 diameter exponents. GLIs are assigned to them, and then 
  * growth is run. Their diameters, heights, and "Growth" values are checked to
  * make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test2Run1YearPerTimestep(clSimManager *p_oSimManager);

  /**
  * Tests the growth when the timestep length is three years. WriteXMLFile4() is
  * used to set up the run. 10 trees are created of three species, assigned to
  * all six growth types and with growth levels both above and below limits. 
  * There are non-1 diameter exponents.  GLIs are assigned to them, and then
  * growth is run.  Their diameters, heights, and "Growth" values are checked to
  * make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test2Run3YearsPerTimestep(clSimManager *p_oSimManager);
  
  /**
  * Tests height growth when the timestep length is one year. WriteXMLFile5() is
  * used to set up the run. 10 trees are created of four species. GLIs are 
  * assigned to them, and then growth is run. Their diameters, heights, and
  * "Growth" values are checked to make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test3Run1YearPerTimestep(clSimManager *p_oSimManager);
  
  /**
  * Tests height growth when the timestep length is three years. WriteXMLFile6()
  * is used to set up the run. 10 trees are created of four species. GLIs are 
  * assigned to them, and then growth is run. Their diameters, heights, and
  * "Growth" values are checked to make sure they are correct.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void Test3Run3YearsPerTimestep(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to direct testing.  Timestep length is 1 year.
  * This contains three species.
  * Behaviors:
  * <ul>
  * <li>Non-limited relative growth is applied to Species 1 saplings.</li>
  * <li>Relative growth limited to constant radial increment is applied to
  * Species 2 saplings.</li>
  * <li>Relative growth limited to basal area increment is applied to Species
  * 1 adults.</li>
  * <li>Non-limited relative growth - diameter only is applied to Species 3
  * saplings.</li>
  * <li>Relative growth limited to constant radial increment - diameter only is
  * applied to Species 4 saplings.</li>
  * <li>Relative growth limited to basal area increment - diameter only is
  * applied to Species 3 adults.</li>
  * <li>Height incrementer applied to all with diameter only behaviors.</li>
  * <li>Diameter exponents are 1.</li>
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a parameter file to direct testing.  Timestep length is 3 years.
  * This contains three species.
  * Behaviors:
  * <ul>
  * <li>Non-limited relative growth is applied to Species 1 saplings.</li>
  * <li>Relative growth limited to constant radial increment is applied to
  * Species 2 saplings.</li>
  * <li>Relative growth limited to basal area increment is applied to Species
  * 1 adults.</li>
  * <li>Non-limited relative growth - diameter only is applied to Species 3
  * saplings.</li>
  * <li>Relative growth limited to constant radial increment - diameter only is
  * applied to Species 4 saplings.</li>
  * <li>Relative growth limited to basal area increment - diameter only is
  * applied to Species 3 adults.</li>
  * <li>Height incrementer applied to all with diameter only behaviors.</li>
  * <li>Diameter exponents are 1.</li>
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile2();

  /**
  * Writes a parameter file to direct testing.  Timestep length is 1 year.
  * This contains three species.
  * Behaviors:
  * <ul>
  * <li>Non-limited relative growth is applied to Species 1 saplings.</li>
  * <li>Relative growth limited to constant radial increment is applied to
  * Species 2 saplings.</li>
  * <li>Relative growth limited to basal area increment is applied to Species
  * 1 adults.</li>
  * <li>Non-limited relative growth - diameter only is applied to Species 3
  * saplings.</li>
  * <li>Relative growth limited to constant radial increment - diameter only is
  * applied to Species 4 saplings.</li>
  * <li>Relative growth limited to basal area increment - diameter only is
  * applied to Species 3 adults.</li>
  * <li>Height incrementer applied to all with diameter only behaviors.</li>
  * <li>Diameter exponents are non-1.</li>
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile3();

  /**
  * Writes a parameter file to direct testing.  Timestep length is 3 years.
  * This contains three species.
  * Behaviors:
  * <ul>
  * <li>Non-limited relative growth is applied to Species 1 saplings.</li>
  * <li>Relative growth limited to constant radial increment is applied to
  * Species 2 saplings.</li>
  * <li>Relative growth limited to basal area increment is applied to Species
  * 1 adults.</li>
  * <li>Non-limited relative growth - diameter only is applied to Species 3
  * saplings.</li>
  * <li>Relative growth limited to constant radial increment - diameter only is
  * applied to Species 4 saplings.</li>
  * <li>Relative growth limited to basal area increment - diameter only is
  * applied to Species 3 adults.</li>
  * <li>Height incrementer applied to all with diameter only behaviors.</li>
  * <li>Diameter exponents are non-1.</li>
  * </ul>
  *
  * @return Filename written.
  */
  char* WriteXMLFile4();
  
  /**
  * Writes a parameter file to direct testing for run 3. Timestep length is 1 
  * year. This contains four species. Relative height growth is applied to all 
  * species. Diameter incrementer applied to all as well.
  *
  * @return Filename written.
  */
  char* WriteXMLFile5();
  
  /**
  * Writes a parameter file to direct testing for run 3. Timestep length is 3 
  * years. This contains four species. Relative height growth is applied to all 
  * species. Diameter incrementer applied to all as well.
  *
  * @return Filename written.
  */
  char* WriteXMLFile6();
  
  /**
  * Writes common portions of the run 3 test parameter files.
  * @param oOut File stream to write to.
  */
  void WriteRun3CommonStuff(std::fstream &oOut);

};
//---------------------------------------------------------------------------
#endif // TestRelativeGrowth_H
