//---------------------------------------------------------------------------
// TestStormKilledPartitionedBiomass
//---------------------------------------------------------------------------
#if !defined(TestStormKilledPartitionedBiomass_H)
  #define TestStormKilledPartitionedBiomass_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;
class clTreePopulation;
class clTree;

/**
* Tests the class clStormKilledPartitionedBiomass.
*/
class clTestStormKilledPartitionedBiomass : public CppUnit::TestFixture {

  public:

  /**
  * Performs all the tests from one function.
  */
  void TestStormKilledPartitionedBiomass();

  protected:

  /**
   * Tests normal processing, run 1.  This uses WriteXMLFile1() to do setup.
   * @param p_oSimManager Sim Manager object to call for runs.
   */
  void TestNormalProcessingRun1(clSimManager *p_oSimManager);
  
  /**
   * Tests normal processing, run 2.  This uses WriteXMLFile2() to do setup.
   * @param p_oSimManager Sim Manager object to call for runs.
   */
  void TestNormalProcessingRun2(clSimManager *p_oSimManager);
  
  /**
   * Tests normal processing, run 3.  This uses WriteXMLFile3() to do setup.
   * @param p_oSimManager Sim Manager object to call for runs.
   */
  void TestNormalProcessingRun3(clSimManager *p_oSimManager);
  
  /**
   * Tests normal processing, run 4.  This uses WriteXMLFile4() to do setup.
   * @param p_oSimManager Sim Manager object to call for runs.
   */
  void TestNormalProcessingRun4(clSimManager *p_oSimManager);
  
  /**
  * Tests error processing.  This uses WriteXMLErrorFileX() to introduce errors
  * and makes sure that they are caught.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes the parameter file for normal processing run 1. 
  * @return Filename written.
  */
  char* WriteXMLFile1();
  
  /**
  * Writes the parameter file for normal processing run 2. 
  * @return Filename written.
  */
  char* WriteXMLFile2();
  
  /**
  * Writes the parameter file for normal processing run 3. 
  * @return Filename written.
  */
  char* WriteXMLFile3();
  
  /**
  * Writes the parameter file for normal processing run 4. 
  * @return Filename written.
  */
  char* WriteXMLFile4();

  /**
  * Writes an error file where the Partitioned Biomass behavior, DBH version,
  * is applied to seedlings.
  * @return Filename written.
  */
  char* WriteXMLErrorFile1();
  
  /**
  * Writes an error file where the Partitioned Biomass behavior, height version,
  * is applied to seedlings.
  * @return Filename written.
  */
  char* WriteXMLErrorFile2();
  
  /**
   * Creates the test tree set.
   * @param p_oPop Tree population object.
   */
  void CreateTrees(clTreePopulation *p_oPop);

  /**
  * Writes common beginning of the test parameter files, which is everything
  * after the grid settings.
  * @param oOut File stream to write to.
  */
  void WriteCommonFileBegin(std::fstream &oOut);
  
  /**
  * Writes common ending of the test parameter files, which is everything
  * after the grid settings.
  * @param oOut File stream to write to.
  */
  void WriteCommonFileEnd(std::fstream &oOut);
};
//---------------------------------------------------------------------------
#endif // TestStormKilledPartitionedBiomass_H
