//---------------------------------------------------------------------------
// TestSubstrate.cpp
//---------------------------------------------------------------------------
#include "TestSubstrate.h"
#include "GridBase.h"
#include "SimManager.h"
#include "BehaviorBase.h"
#include "TreePopulation.h"
#include <math.h>
#include <fstream>

/////////////////////////////////////////////////////////////////////////////
//  TestSubstrate()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::TestSubstrate()
{
  clSimManager * p_oSimManager = new clSimManager(3, 3, "");
  TestSubstrateGridSetup(p_oSimManager);
  TestTreeFall(p_oSimManager);
  TestHarvestSubstrate(p_oSimManager);
  TestSubstrateRun1(p_oSimManager);
  TestSubstrateRun2(p_oSimManager);
  TestSubstrateRun3(p_oSimManager);
  TestSubstrateRun4(p_oSimManager);
  delete p_oSimManager;
}

/////////////////////////////////////////////////////////////////////////////
//  TestTreeFall()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::TestTreeFall(clSimManager *p_oSimManager)
{
  clPopulationBase *p_oTemp;
  clTreePopulation *p_oPop;
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  int iNumSp1Snags = 0, iNumSp2Snags = 0, iNumSp3Snags = 0,
      iDead = natural,
      iSpecies, i;

  //Create and feed in the file
  p_oSimManager->ReadFile(WriteXMLFile5());
  //Grab our pointers
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation*) p_oTemp;

  //Create 1000 of each species of tree, all adults
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 1, 0, clTreePopulation::adult, 17);
  }
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 1, 1, clTreePopulation::adult, 17);
  }
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 1, 2, clTreePopulation::adult, 17);
  }
  //Create 1000 of each species of tree, all snags
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(2, 1, 0, clTreePopulation::snag, 17);
    p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);
  }
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(2, 1, 1, clTreePopulation::snag, 17);
    p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);
  }
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(2, 1, 2, clTreePopulation::snag, 17);
    p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);
  }

  //Run for 1 timestep and make sure all the snags have fallen
  p_oSimManager->RunSim(1);

  //Count up the snags and make sure there's the right number of each
  iNumSp1Snags = 0;
  iNumSp2Snags = 0;
  iNumSp3Snags = 0;
  p_oAllTrees = p_oPop->Find("all");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Make sure it's not alive
    CPPUNIT_ASSERT(clTreePopulation::snag == p_oTree->GetType());

    //Count it
    iSpecies = p_oTree->GetSpecies();
    if (0 == iSpecies) iNumSp1Snags++;
    else if (1 == iSpecies) iNumSp2Snags++;
    else if (2 == iSpecies) iNumSp3Snags++;

    p_oTree = p_oAllTrees->NextTree();
  }

  //Now make sure we have the number we expect left standing
  CPPUNIT_ASSERT(fabs(iNumSp1Snags - 800)/800 < 0.1);
  CPPUNIT_ASSERT(fabs(iNumSp2Snags - 500)/500 < 0.1);
  CPPUNIT_ASSERT(iNumSp3Snags == 1000);

  //Run another timestep and count them again
  iNumSp1Snags = 0;
  iNumSp2Snags = 0;
  iNumSp3Snags = 0;
  p_oSimManager->RunSim(1);
  p_oAllTrees = p_oPop->Find("all");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Make sure it's not alive
    CPPUNIT_ASSERT(clTreePopulation::snag == p_oTree->GetType());

    //Count it
    iSpecies = p_oTree->GetSpecies();
    if (0 == iSpecies) iNumSp1Snags++;
    else if (1 == iSpecies) iNumSp2Snags++;
    else if (2 == iSpecies) iNumSp3Snags++;

    p_oTree = p_oAllTrees->NextTree();
  }

  //Now make sure we have the number we expect left standing
  CPPUNIT_ASSERT(fabs(iNumSp1Snags - 800)/800 < 0.1);
  CPPUNIT_ASSERT(fabs(iNumSp2Snags - 500)/500 < 0.1);
  CPPUNIT_ASSERT(iNumSp3Snags == 1000);

  //***************************
  // Test the tipup
  //***************************
  p_oSimManager->ReadFile(WriteXMLFile4());
  //Grab our pointers
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  clGridBase *p_oCalcsGrid = p_oSimManager->GetGridObject("substratecalcs");
  p_oPop = (clTreePopulation*) p_oTemp;

  //Run for 1 timestep
  p_oSimManager->RunSim(1);

  //Create 1000 of each species of tree, all adults
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 1, 0, clTreePopulation::adult, 20);
  }
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 46, 1, clTreePopulation::adult, 40);
  }
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 91, 2, clTreePopulation::adult, 55);
  }
  //Create 1000 of each species of tree, all snags
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 135, 0, clTreePopulation::snag, 25);
  }
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 181, 1, clTreePopulation::snag, 30);
  }
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 226, 2, clTreePopulation::snag, 65);
  }

  //Run for 1 timestep
  p_oSimManager->RunSim(1);

  float fExpectedTipup, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newtipup"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedTipup = 0.149089903;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup)/fExpectedTipup < 0.1);
  //Cell 2
  iY = 1;
  fExpectedTipup = 0.097488417;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup)/fExpectedTipup < 0.1);
  //Cell 3
  iY = 2;
  fExpectedTipup = 0.049159992;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup)/fExpectedTipup < 0.2);
  //Cell 4
  iY = 3;
  fExpectedTipup = 0.123516502;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup)/fExpectedTipup < 0.1);
  //Cell 5
  iY = 4;
  fExpectedTipup = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fActual == fExpectedTipup);
  //Cell 6
  iY = 5;
  fExpectedTipup = 0.147523047;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup)/fExpectedTipup < 0.1);
}

/////////////////////////////////////////////////////////////////////////////
//  TestSubstrateRun1()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::TestSubstrateRun1(clSimManager *p_oSimManager)
{
  clGridBase * p_oSubstrateGrid, *p_oCalcsGrid;
  clTreePopulation *p_oPop;
  clTree *p_oTree;
  int iDead = natural;

  //Create and feed in the file
  p_oSimManager->ReadFile(WriteXMLFile2());
  //Grab our pointers
  p_oPop = (clTreePopulation*) p_oSimManager->GetPopulationObject("treepopulation");
  p_oSubstrateGrid = p_oSimManager->GetGridObject("Substrate");
  p_oCalcsGrid = p_oSimManager->GetGridObject("substratecalcs");

  //********************************
  //Timestep 1:  Just run.
  //********************************
  p_oSimManager->RunSim(1);

  //Verify the freshlogs and declogs arrays
  VerifyRun1TS1SubstrateArrays(p_oCalcsGrid);

  //Verify the new tipups
  VerifyRun1TS1Tipup(p_oCalcsGrid);

  //Verify the cohorts
  VerifyRun1TS1Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyRun1TS1Proportions(p_oSubstrateGrid);

  //********************************
  //Timestep 2:
  //********************************
  //Create and then kill certain trees within cell 0, 2
  //Tree where y = 18
  p_oTree = p_oPop->CreateTree(1, 18, 0, 3, 20);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Tree where y = 19
  p_oTree = p_oPop->CreateTree(1, 19, 0, 3, 40);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);


  //Now create and then kill certain trees within cell 0, 3
  //Tree where y = 25
  p_oTree = p_oPop->CreateTree(1, 25, 0, 3, 55);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Tree where y = 26
  p_oTree = p_oPop->CreateTree(1, 26, 0, 3, 15);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Tree where y = 27
  p_oTree = p_oPop->CreateTree(1, 27, 0, 2, 8);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  p_oSimManager->RunSim(1);


  //Verify the freshlogs and declogs arrays
  VerifyRun1TS2SubstrateArrays(p_oCalcsGrid);

  //Verify the new tipups
  VerifyRun1TS2Tipup(p_oCalcsGrid);

  //Verify the cohorts
  VerifyRun1TS2Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyRun1TS2Proportions(p_oSubstrateGrid);

  //********************************
  //Timestep 3:
  //********************************
  //Create and kill a tree in cell 0, 0 where y = 2
  p_oTree = p_oPop->CreateTree(1, 2, 0, 2, 65);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Create and kill a tree in cell 0, 1 where y = 10
  p_oTree = p_oPop->CreateTree(1, 10, 0, 2, 100);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Create and kill a tree in cell 0, 2 where y = 20
  p_oTree = p_oPop->CreateTree(1, 20, 0, 2, 35);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Create and kill a tree in cell 0, 3 where y = 28
  p_oTree = p_oPop->CreateTree(1, 28, 0, 2, 42);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Now run the harvest behavior, substrate behavior and the tree killer
  p_oSimManager->RunSim(1);

  //Verify the freshlogs and declogs arrays
  VerifyRun1TS3SubstrateArrays(p_oCalcsGrid);

  //Verify the new tipups
  VerifyRun1TS3Tipup(p_oCalcsGrid);

  //Verify the cohorts
  VerifyRun1TS3Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyRun1TS3Proportions(p_oSubstrateGrid);

  //********************************
  //Timestep 4:  Just run.
  //********************************
  p_oSimManager->RunSim(1);

  //Verify the freshlogs and declogs arrays
  VerifyRun1TS4SubstrateArrays(p_oCalcsGrid);

  //Verify the new tipups
  VerifyRun1TS4Tipup(p_oCalcsGrid);

  //Verify the cohorts
  VerifyRun1TS4Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyRun1TS4Proportions(p_oSubstrateGrid);
}

/////////////////////////////////////////////////////////////////////////////
//  TestSubstrateRun2()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::TestSubstrateRun2(clSimManager *p_oSimManager)
{
  clGridBase * p_oSubstrateGrid, *p_oCalcsGrid;
  clPopulationBase *p_oTemp;
  clTreePopulation *p_oPop;
  clTree *p_oTree;
  int i, iDead = natural;

  //Create and feed in the file
  p_oSimManager->ReadFile(WriteXMLFile1());
  //Grab our pointers
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation*) p_oTemp;
  p_oSubstrateGrid = p_oSimManager->GetGridObject("Substrate");
  p_oCalcsGrid = p_oSimManager->GetGridObject("substratecalcs");

  //Create our trees for harvesting
  p_oTree = p_oPop->CreateTree(1, 1, 0, clTreePopulation::adult, 20);
  p_oTree = p_oPop->CreateTree(1, 9, 0, clTreePopulation::adult, 20);
  p_oTree = p_oPop->CreateTree(1, 17, 0, clTreePopulation::adult, 20);

  //********************************
  //Timestep 1:  Just run.
  //********************************
  p_oSimManager->RunSim(1);

  //Verify the freshlogs and declogs arrays
  VerifyRun2TS1SubstrateArrays(p_oCalcsGrid);

  //Verify the new tipups
  VerifyRun2TS1Tipup(p_oCalcsGrid);

  //Verify the cohorts
  VerifyRun2TS1Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyRun2TS1Proportions(p_oSubstrateGrid);

  //********************************
  //Timestep 2:
  //********************************

  //Set the timestep to 2
  p_oSimManager->SetCurrentTimestep(2);

  //Run the harvest behavior
  p_oSimManager->GetBehaviorObject(0)->Action();

  //Create and then kill certain trees within cell 0, 2
  //Tree where y = 18
  p_oTree = p_oPop->CreateTree(1, 18, 0, 3, 20);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Tree where y = 19
  p_oTree = p_oPop->CreateTree(1, 19, 0, 3, 40);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);


  //Now create and then kill certain trees within cell 0, 3
  //Tree where y = 25
  p_oTree = p_oPop->CreateTree(1, 25, 0, 3, 55);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Tree where y = 26
  p_oTree = p_oPop->CreateTree(1, 26, 0, 3, 15);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Tree where y = 27
  p_oTree = p_oPop->CreateTree(1, 27, 0, 2, 8);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);


  //Now run the substrate behavior and the tree killer
  for (i = 2; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    p_oSimManager->GetBehaviorObject(i)->Action();
  }
  //Now timestep cleanup
  for (i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    p_oSimManager->GetBehaviorObject(i)->TimestepCleanup();
  }

  //Verify the freshlogs and declogs arrays
  VerifyRun2TS2SubstrateArrays(p_oCalcsGrid);

  //Verify the new tipups
  VerifyRun2TS2Tipup(p_oCalcsGrid);

  //Verify the cohorts
  VerifyRun2TS2Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyRun2TS2Proportions(p_oSubstrateGrid);

  //********************************
  //Timestep 3:
  //********************************
  //We have to fake this timestep.  The light behavior will interfere with our
  //ability to kill trees, so we have to skip it.  Set the timestep to 3
  p_oSimManager->SetCurrentTimestep(3);

  //Create and kill a tree in cell 0, 0 where y = 2
  p_oTree = p_oPop->CreateTree(1, 2, 0, 2, 65);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Create and kill a tree in cell 0, 1 where y = 10
  p_oTree = p_oPop->CreateTree(1, 10, 0, 2, 100);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Create and kill a tree in cell 0, 2 where y = 20
  p_oTree = p_oPop->CreateTree(1, 20, 0, 2, 35);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Create and kill a tree in cell 0, 3 where y = 28
  p_oTree = p_oPop->CreateTree(1, 28, 0, 2, 42);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Now run the harvest behavior, substrate behavior and the tree killer
  p_oSimManager->GetBehaviorObject(0)->Action();
  for (i = 2; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    p_oSimManager->GetBehaviorObject(i)->Action();
  }
  //Now timestep cleanup
  for (i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    p_oSimManager->GetBehaviorObject(i)->TimestepCleanup();
  }

  //Verify the freshlogs and declogs arrays
  VerifyRun2TS3SubstrateArrays(p_oCalcsGrid);

  //Verify the new tipups
  VerifyRun2TS3Tipup(p_oCalcsGrid);

  //Verify the cohorts
  VerifyRun2TS3Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyRun2TS3Proportions(p_oSubstrateGrid);

  //********************************
  //Timestep 4:  Just run.
  //********************************
  p_oSimManager->RunSim(1);

  //Verify the freshlogs and declogs arrays
  VerifyRun2TS4SubstrateArrays(p_oCalcsGrid);

  //Verify the new tipups
  VerifyRun2TS4Tipup(p_oCalcsGrid);

  //Verify the cohorts
  VerifyRun2TS4Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyRun2TS4Proportions(p_oSubstrateGrid);
}

/////////////////////////////////////////////////////////////////////////////
//  TestSubstrateRun3()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::TestSubstrateRun3(clSimManager *p_oSimManager)
{
  clGridBase * p_oSubstrateGrid, *p_oCalcsGrid;
  clTreePopulation *p_oPop;
  clTree *p_oTree;
  char cLabel[20];
  int *p_iFreshLogCodes = new int[3];
  float fExpectedScarSoil = 0, fExpectedFFMoss, fExpectedFFLitter, fExpectedTipup,
  fExpectedDecLog, fExpectedFreshLog, fActual;
  int iX, iY, i, j, iCode;


  //Create and feed in the file
  p_oSimManager->ReadFile(WriteXMLFile3());
  //Grab our pointers
  p_oPop = (clTreePopulation*) p_oSimManager->GetPopulationObject("treepopulation");
  p_oSubstrateGrid = p_oSimManager->GetGridObject("Substrate");
  p_oCalcsGrid = p_oSimManager->GetGridObject("substratecalcs");

  //********************************
  //Timestep 1:  Run.
  //********************************
  p_oSimManager->RunSim(1);

  //Verify the freshlogs and declogs arrays
  fExpectedFreshLog = 0.01; //all values are the same
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iFreshLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iFreshLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 5; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iX = 5; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iX = 15; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iX = 15; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Last timestep
  fExpectedFreshLog = 0.0;
  //Cell 1
  iX = 5; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iX = 5; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iX = 15; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iX = 15; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 5; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iX = 5; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iX = 15; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iX = 15; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Verify the new tipups
  //Cell 1
  fExpectedTipup = 0.2;
  iCode = p_oCalcsGrid->GetFloatDataCode("newtipup");
  iX = 5; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 2
  iX = 5; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 3
  iX = 15; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 4
  iX = 15; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  //Verify the substrate proportions
  fExpectedFreshLog = 0.01;
  fExpectedDecLog = 0.11;
  fExpectedTipup = 0.2;
  fExpectedFFMoss = 0.238;
  fExpectedFFLitter = 0.442;
  iX = 5; iY = 5;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iX = 5; iY = 15;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iX = 15; iY = 5;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iX = 15; iY = 15;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //********************************
  //Timestep 2:
  //********************************
  //Create certain trees within each test grid cell - these should all get killed
  p_oTree = p_oPop->CreateTree(44, 44, 0, clTreePopulation::adult, 20);
  p_oTree = p_oPop->CreateTree(44, 124, 0, clTreePopulation::adult, 30);
  p_oTree = p_oPop->CreateTree(124, 44, 0, clTreePopulation::adult, 35);
  p_oTree = p_oPop->CreateTree(124, 124, 0, clTreePopulation::adult, 40);

  p_oSimManager->RunSim(1);

  //Current timestep addition - make sure these do NOT equal
  //Cell 1
  iX = 5; iY = 5;
  fExpectedFreshLog = 0.028924872;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog)/fExpectedFreshLog > 0.1);
  //Cell 2
  iX = 5; iY = 15;
  fExpectedFreshLog = 0.056088097;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog)/fExpectedFreshLog > 0.1);
  //Cell 3
  iX = 15; iY = 5;
  fExpectedFreshLog = 0.071340092;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog)/fExpectedFreshLog > 0.1);
  //Cell 4
  iX = 15; iY = 15;
  fExpectedFreshLog = 0.087341983;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog)/fExpectedFreshLog > 0.1);

  //Now add up each target grid cell and all surrounding cells - they should
  //total the amount in each tree trunk
  //Cell 1
  fActual = 0;
  fExpectedFreshLog = 0.028924872;
  iX = 5; iY = 5;
  for (i = iX - 3; i <= iX + 3; i++) {
    for (j = iY - 3; j <= iY + 3; j++) {
      //Dangerously co-opt this variable
       p_oCalcsGrid->GetValueOfCell(i, j, p_iFreshLogCodes[0], & fExpectedTipup);
       fActual += fExpectedTipup;
    }
  }
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Cell 2
  fActual = 0;
  fExpectedFreshLog = 0.056088097;
  iX = 5; iY = 15;
  for (i = iX - 3; i <= iX + 3; i++) {
    for (j = iY - 3; j <= iY + 3; j++) {
      //Dangerously co-opt this variable
       p_oCalcsGrid->GetValueOfCell(i, j, p_iFreshLogCodes[0], & fExpectedTipup);
       fActual += fExpectedTipup;
    }
  }
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Cell 3
  fActual = 0;
  fExpectedFreshLog = 0.071340092;
  iX = 15; iY = 5;
  for (i = iX - 3; i <= iX + 3; i++) {
    for (j = iY - 3; j <= iY + 3; j++) {
      //Dangerously co-opt this variable
       p_oCalcsGrid->GetValueOfCell(i, j, p_iFreshLogCodes[0], & fExpectedTipup);
       fActual += fExpectedTipup;
    }
  }
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Cell 4
  fActual = 0;
  fExpectedFreshLog = 0.087341983;
  iX = 15; iY = 15;
  for (i = iX - 3; i <= iX + 3; i++) {
    for (j = iY - 3; j <= iY + 3; j++) {
      //Dangerously co-opt this variable
       p_oCalcsGrid->GetValueOfCell(i, j, p_iFreshLogCodes[0], & fExpectedTipup);
       fActual += fExpectedTipup;
    }
  }
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Last timestep
  fExpectedFreshLog = 0.01;
  //Cell 1
  iX = 5; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iX = 5; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iX = 15; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iX = 15; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Timestep - 2
  fExpectedFreshLog = 0.0;
  //Cell 1
  iX = 5; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iX = 5; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iX = 15; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iX = 15; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Verify the new tipups
  //Cell 1
  fExpectedTipup = 0.004717298;
  iCode = p_oCalcsGrid->GetFloatDataCode("newtipup");
  iX = 5; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 2
  fExpectedTipup = 0.01061392;
  iX = 5; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 3
  fExpectedTipup = 0.014446724;
  iX = 15; iY = 5;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 4
  fExpectedTipup = 0.018869191;
  iX = 15; iY = 15;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  //Don't verify the proportions - too complicated

  delete[] p_iFreshLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  TestSubstrateRun4()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::TestSubstrateRun4(clSimManager *p_oSimManager)
{
  clGridBase * p_oSubstrateGrid, *p_oCalcsGrid;
  clPopulationBase *p_oTemp;
  clTreePopulation *p_oPop;
  clTree *p_oTree;
  int iDead = natural;

  //Create and feed in the file
  p_oSimManager->ReadFile(WriteXMLFile6());
  //Grab our pointers
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation*) p_oTemp;
  p_oSubstrateGrid = p_oSimManager->GetGridObject("Substrate");
  p_oCalcsGrid = p_oSimManager->GetGridObject("substratecalcs");

  //********************************
  //Timestep 1:  Just run.
  //********************************
  p_oSimManager->RunSim(1);

  //Verify the freshlogs and declogs arrays
  VerifyRun4TS1SubstrateArrays(p_oCalcsGrid);

  //Verify the new tipups
  VerifyRun1TS1Tipup(p_oCalcsGrid);

  //Verify the cohorts
  VerifyRun1TS1Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyRun4TS1Proportions(p_oSubstrateGrid);

  //********************************
  //Timestep 2:
  //********************************
  //Create and then kill certain trees within cell 0, 2
  //Tree where y = 18
  p_oTree = p_oPop->CreateTree(1, 18, 0, 3, 20);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Tree where y = 19
  p_oTree = p_oPop->CreateTree(1, 19, 0, 3, 40);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);


  //Now create and then kill certain trees within cell 0, 3
  //Tree where y = 25
  p_oTree = p_oPop->CreateTree(1, 25, 0, 3, 55);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Tree where y = 26
  p_oTree = p_oPop->CreateTree(1, 26, 0, 3, 15);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Tree where y = 27
  p_oTree = p_oPop->CreateTree(1, 27, 0, 2, 8);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  p_oSimManager->RunSim(1);


  //Verify the freshlogs and declogs arrays
  VerifyRun4TS2SubstrateArrays(p_oCalcsGrid);

  //Verify the new tipups
  VerifyRun1TS2Tipup(p_oCalcsGrid);

  //Verify the cohorts
  VerifyRun1TS2Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyRun4TS2Proportions(p_oSubstrateGrid);

  //********************************
  //Timestep 3:
  //********************************
  //Create and kill a tree in cell 0, 0 where y = 2
  p_oTree = p_oPop->CreateTree(1, 2, 0, 2, 65);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Create and kill a tree in cell 0, 1 where y = 10
  p_oTree = p_oPop->CreateTree(1, 10, 0, 2, 100);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Create and kill a tree in cell 0, 2 where y = 20
  p_oTree = p_oPop->CreateTree(1, 20, 0, 2, 35);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Create and kill a tree in cell 0, 3 where y = 28
  p_oTree = p_oPop->CreateTree(1, 28, 0, 2, 42);
  p_oTree->SetValue(p_oPop->GetIntDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), iDead);

  //Now run the harvest behavior, substrate behavior and the tree killer
  p_oSimManager->RunSim(1);

  //Verify the freshlogs and declogs arrays
  VerifyRun4TS3SubstrateArrays(p_oCalcsGrid);

  //Verify the new tipups
  VerifyRun1TS3Tipup(p_oCalcsGrid);

  //Verify the cohorts
  VerifyRun1TS3Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyRun4TS3Proportions(p_oSubstrateGrid);

  //********************************
  //Timestep 4:  Just run.
  //********************************
  p_oSimManager->RunSim(1);

  //Verify the freshlogs and declogs arrays
  VerifyRun4TS4SubstrateArrays(p_oCalcsGrid);

  //Verify the new tipups
  VerifyRun1TS4Tipup(p_oCalcsGrid);

  //Verify the cohorts
  VerifyRun1TS4Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyRun4TS4Proportions(p_oSubstrateGrid);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS1Proportions()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS1Proportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil,
        fExpectedDecLog, fExpectedFFMoss, fExpectedFFLitter, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedScarSoil = 0;
  fExpectedFreshLog = 0.01;
  fExpectedDecLog = 0.11;
  fExpectedTipup = 0.2;
  fExpectedFFMoss = 0.476;
  fExpectedFFLitter = 0.204;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iY = 1;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iY = 2;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iY = 3;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 5
  iY = 4;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS2Proportions()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS2Proportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil,
        fExpectedDecLog, fExpectedFFMoss, fExpectedFFLitter, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedScarSoil = 0.45;
  fExpectedFreshLog = 0.09;
  fExpectedDecLog = 0;
  fExpectedTipup = 0.34;
  fExpectedFFMoss = 0.084;
  fExpectedFFLitter = 0.036;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedScarSoil = 0.36;
  fExpectedFreshLog = 0.17;
  fExpectedDecLog = 0.01;
  fExpectedTipup = 0.28;
  fExpectedFFMoss = 0.126;
  fExpectedFFLitter = 0.054;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedScarSoil = 0.146224932;
  fExpectedFreshLog = 0.167875654;
  fExpectedDecLog = 0.04;
  fExpectedTipup = 0.152608487;
  fExpectedFFMoss = 0.345303649;
  fExpectedFFLitter = 0.147987278;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedScarSoil = 0;
  fExpectedFreshLog = 0.169256747;
  fExpectedDecLog = 0.049988087;
  fExpectedTipup = 0.198221137;
  fExpectedFFMoss = 0.407773821;
  fExpectedFFLitter = 0.174760209;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedScarSoil = 0;
  fExpectedFreshLog = 0.009512294;
  fExpectedDecLog = 0.049988087;
  fExpectedTipup = 0.199860049;
  fExpectedFFMoss = 0.518447699;
  fExpectedFFLitter = 0.222191871;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS3Proportions()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS3Proportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil,
        fExpectedDecLog, fExpectedFFMoss, fExpectedFFLitter, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedScarSoil = 0.349659363;
  fExpectedFreshLog = 0.239311544;
  fExpectedDecLog = 0.004389352;
  fExpectedTipup = 0.313963341;
  fExpectedFFMoss = 0.06487348;
  fExpectedFFLitter = 0.02780292;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedScarSoil = 0.211791635;
  fExpectedFreshLog = 0.388640914;
  fExpectedDecLog = 0.012791032;
  fExpectedTipup = 0.282627975;
  fExpectedFFMoss = 0.07290391;
  fExpectedFFLitter = 0.031244533;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedScarSoil = 0.1336126;
  fExpectedFreshLog = 0.217329207;
  fExpectedDecLog = 0.026596774;
  fExpectedTipup = 0.153865788;
  fExpectedFFMoss = 0.328016942;
  fExpectedFFLitter = 0.140578689;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedScarSoil = 0;
  fExpectedFreshLog = 0.230107105;
  fExpectedDecLog = 0.037370059;
  fExpectedTipup = 0.194422339;
  fExpectedFFMoss = 0.376670348;
  fExpectedFFLitter = 0.161430149;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedScarSoil = 0;
  fExpectedFreshLog = 0.000108295;
  fExpectedDecLog = 0.029486458;
  fExpectedTipup = 0.197261163;
  fExpectedFFMoss = 0.541200859;
  fExpectedFFLitter = 0.231943225;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS4Proportions()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS4Proportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil,
        fExpectedDecLog, fExpectedFFMoss, fExpectedFFLitter, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedScarSoil = 0.346091173;
  fExpectedFreshLog = 0.16508864;
  fExpectedDecLog = 0.095036646;
  fExpectedTipup = 0.310493762;
  fExpectedFFMoss = 0.058302845;
  fExpectedFFLitter = 0.024986934;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedScarSoil = 0.209630353;
  fExpectedFreshLog = 0.280228899;
  fExpectedDecLog = 0.179716778;
  fExpectedTipup = 0.280403828;
  fExpectedFFMoss = 0.0350141;
  fExpectedFFLitter = 0.015006043;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedScarSoil = 0.132249116;
  fExpectedFreshLog = 0.06952284;
  fExpectedDecLog = 0.180332059;
  fExpectedTipup = 0.152042738;
  fExpectedFFMoss = 0.326097273;
  fExpectedFFLitter = 0.139755974;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedScarSoil = 0;
  fExpectedFreshLog = 0.090875207;
  fExpectedDecLog = 0.164459882;
  fExpectedTipup = 0.054255551;
  fExpectedFFMoss = 0.483286552;
  fExpectedFFLitter = 0.207122808;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedScarSoil = 0;
  fExpectedFreshLog = 0.000108295;
  fExpectedDecLog = 0.004319899;
  fExpectedTipup = 0.197261163;
  fExpectedFFMoss = 0.558817451;
  fExpectedFFLitter = 0.239493193;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS1Cohorts()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS1Cohorts(clGridBase *p_oSubstrateGrid) {
  clPackage * p_oCohort;
  float fExpectedTipup = 0.2, fExpectedFreshLog = 0.01, fExpectedScarSoil = 0, fActual;
  int iX, iY, iAge = 1, iActual;

  //Cell 1
  iX = 0; iY = 0;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 2
  iY = 1;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 3
  iY = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 4
  iY = 3;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 5
  iY = 4;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS2Cohorts()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS2Cohorts(clGridBase *p_oSubstrateGrid) {
  clPackage * p_oCohort;
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil, fActual;
  int iX, iY, iAge, iActual;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedTipup = 0.34;
  fExpectedFreshLog = 0.09;
  fExpectedScarSoil = 0.45;
  iAge = 1;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 2
  iY = 1;
  fExpectedTipup = 0.28;
  fExpectedFreshLog = 0.17;
  fExpectedScarSoil = 0.36;
  iAge = 1;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 3
  iY = 2;
  fExpectedTipup = 0.152608487;
  fExpectedFreshLog = 0.167875654;
  fExpectedScarSoil = 0.146224932;
  iAge = 1;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 4, cohort 1
  iY = 3;
  fExpectedTipup = 0.160005057;
  fExpectedFreshLog = 0.008000253;
  fExpectedScarSoil = 0;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  //Cell 4, cohort 2
  fExpectedTipup = 0.038328044;
  fExpectedFreshLog = 0.161646671;
  fExpectedScarSoil = 0;
  iAge = 1;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 5
  iY = 4;
  fExpectedTipup = 0.2;
  fExpectedFreshLog = 0.01;
  fExpectedScarSoil = 0;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS3Cohorts()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS3Cohorts(clGridBase *p_oSubstrateGrid) {
  clPackage * p_oCohort;
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil, fActual;
  int iX, iY, iAge, iActual;

  //Cell 1, cohort 1
  iX = 0; iY = 0;
  fExpectedTipup = 0.264321844;
  fExpectedFreshLog = 0.069967547;
  fExpectedScarSoil = 0.349837735;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  //Cell 1, cohort 2
  fExpectedTipup = 0.049826457;
  fExpectedFreshLog = 0.172756354;
  fExpectedScarSoil = 0;
  iAge = 1;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 2, cohort 1
  iY = 1;
  fExpectedTipup = 0.16481086;
  fExpectedFreshLog = 0.100063736;
  fExpectedScarSoil = 0.211899677;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  //Cell 2, cohort 2
  fExpectedTipup = 0.117932443;
  fExpectedFreshLog = 0.293457344;
  fExpectedScarSoil = 0;
  iAge = 1;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());


  //Cell 3, cohort 1
  iY = 2;
  fExpectedTipup = 0.139516691;
  fExpectedFreshLog = 0.153474136;
  fExpectedScarSoil = 0.13368076;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  //Cell 3, cohort 2
  fExpectedTipup = 0.014446724;
  fExpectedFreshLog = 0.071340092;
  fExpectedScarSoil = 0;
  iAge = 1;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 4, cohort 1
  iY = 3;
  fExpectedTipup = 0.033931489;
  fExpectedFreshLog = 0.143104415;
  fExpectedScarSoil = 0;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  //Cell 4, cohort 2
  fExpectedTipup = 0.020803283;
  fExpectedFreshLog = 0.093905275;
  fExpectedScarSoil = 0;
  iAge = 1;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 5
  iY = 4;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL == p_oCohort);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS4Cohorts()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS4Cohorts(clGridBase *p_oSubstrateGrid) {
  clPackage * p_oCohort;
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil, fActual;
  int iX, iY, iAge, iActual;

  //Cell 1, cohort 1
  iX = 0; iY = 0;
  fExpectedTipup = 0.049826457;
  fExpectedFreshLog = 0.172756354;
  fExpectedScarSoil = 0;
  iAge = 2;

  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);
  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 2, cohort 1
  iY = 1;
  fExpectedTipup = 0.117932443;
  fExpectedFreshLog = 0.293457344;
  fExpectedScarSoil = 0;
  iAge = 2;

  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 3, cohort 1
  iY = 2;
  fExpectedTipup = 0.014446724;
  fExpectedFreshLog = 0.071340092;
  fExpectedScarSoil = 0;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 4, cohort 1
  iY = 3;
  fExpectedTipup = 0.020803283;
  fExpectedFreshLog = 0.093905275;
  fExpectedScarSoil = 0;
  iAge = 2;

  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 5
  iY = 4;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL == p_oCohort);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS1Tipup()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS1Tipup(clGridBase *p_oCalcsGrid) {

  float fExpectedTipup = 0.2, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newtipup"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS2Tipup()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS2Tipup(clGridBase *p_oCalcsGrid) {

  float fExpectedTipup, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newtipup"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedTipup = 0.34;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedTipup = 0.28;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedTipup = 0.173586489;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedTipup = 0.038328044;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedTipup = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS3Tipup()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS3Tipup(clGridBase *p_oCalcsGrid) {

  float fExpectedTipup, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newtipup"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedTipup = 0.049826457;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedTipup = 0.117932443;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedTipup = 0.014446724;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedTipup = 0.020803283;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedTipup = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS4Tipup()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS4Tipup(clGridBase *p_oCalcsGrid) {

  float fExpectedTipup = 0.0, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newtipup"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS1SubstrateArrays()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS1SubstrateArrays(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedLog = 0.01, //all values are the same
        fActual;
  int *p_iLogCodes = new int[3],
       iX, iY, i;

  //Fresh logs
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  fExpectedLog = 0.0;
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Decayed logs
  fExpectedLog = 0.11;
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "declog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  fExpectedLog = 0.0;
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  delete[] p_iLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS2SubstrateArrays()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS2SubstrateArrays(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedLog, //all values are the same
        fActual;
  int *p_iLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.09;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.17;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.176266855;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.161646671;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.01;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.01;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  fExpectedLog = 0.0;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Decayed logs
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "declog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.01;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.04;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  delete[] p_iLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS3SubstrateArrays()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS3SubstrateArrays(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedLog, //all values are the same
        fActual;
  int *p_iLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.172756354;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.293457344;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.071340092;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.093905275;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.09;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.17;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.176266855;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.161646671;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  fExpectedLog = 0.0;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  fExpectedLog = 0.0;
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  fExpectedLog = 0.0;
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  fExpectedLog = 0.01;
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  fExpectedLog = 0.01;
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Decayed logs
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "declog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.004389352;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.008290998;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.008596636;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.017287601;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.009403999;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.01;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.04;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  delete[] p_iLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun2TS4SubstrateArrays()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun2TS4SubstrateArrays(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedLog, //all values are the same
        fActual;
  int *p_iLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  fExpectedLog = 0.0;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  iX = 0; iY = 0;
  fExpectedLog = 0.172756354;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.293457344;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.071340092;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.093905275;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.09;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.17;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.176266855;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.161646671;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Decayed logs
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "declog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.093061422;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.174180074;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.169240638;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.156592335;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.004389352;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.008290998;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.008596636;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.017287601;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.009403999;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.01;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.04;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  delete[] p_iLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS1Proportions()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS1Proportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil,
        fExpectedDecLog, fExpectedFFMoss, fExpectedFFLitter, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedScarSoil = 0;
  fExpectedFreshLog = 0.01;
  fExpectedDecLog = 0.11;
  fExpectedTipup = 0.2;
  fExpectedFFMoss = 0.238;
  fExpectedFFLitter = 0.442;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iY = 1;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iY = 2;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iY = 3;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 5
  iY = 4;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS2Proportions()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS2Proportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil = 0,
        fExpectedDecLog, fExpectedFFMoss, fExpectedFFLitter, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.009512294;
  fExpectedDecLog = 0.049988087;
  fExpectedTipup = 0.199860049;
  fExpectedFFMoss = 0.25922385;
  fExpectedFFLitter = 0.481415721;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.009512294;
  fExpectedDecLog = 0.049988087;
  fExpectedTipup = 0.199860049;
  fExpectedFFLitter = 0.481415721;
  fExpectedFFMoss = 0.25922385;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.124448823;
  fExpectedDecLog = 0.049988087;
  fExpectedTipup = 0.195495441;
  fExpectedFFMoss = 0.220523677;
  fExpectedFFLitter = 0.409543972;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.163399328;
  fExpectedDecLog = 0.049988087;
  fExpectedTipup = 0.199403043;
  fExpectedFFMoss = 0.20552334;
  fExpectedFFLitter = 0.381686202;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0.009512294;
  fExpectedDecLog = 0.049988087;
  fExpectedTipup = 0.199860049;
  fExpectedFFMoss = 0.25922385;
  fExpectedFFLitter = 0.481415721;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS3Proportions()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS3Proportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil = 0,
        fExpectedDecLog, fExpectedFFMoss, fExpectedFFLitter, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.172840545;
  fExpectedDecLog = 0.031898812;
  fExpectedTipup = 0.203180676;
  fExpectedFFMoss = 0.207227989;
  fExpectedFFLitter = 0.384851979;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.293521087;
  fExpectedDecLog = 0.031898812;
  fExpectedTipup = 0.234042378;
  fExpectedFFMoss = 0.154188203;
  fExpectedFFLitter = 0.28634952;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.172533986;
  fExpectedDecLog = 0.037569213;
  fExpectedTipup = 0.191112492;
  fExpectedFFMoss = 0.209574508;
  fExpectedFFLitter = 0.389209801;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.225127679;
  fExpectedDecLog = 0.039494;
  fExpectedTipup = 0.195455065;
  fExpectedFFMoss = 0.18897314;
  fExpectedFFLitter = 0.350950116;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0.000108295;
  fExpectedDecLog = 0.031898812;
  fExpectedTipup = 0.197261163;
  fExpectedFFMoss = 0.269756106;
  fExpectedFFLitter = 0.500975625;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS4Proportions()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS4Proportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil,
        fExpectedDecLog, fExpectedFFMoss, fExpectedFFLitter, fActual;
  int iX, iY;

  fExpectedScarSoil = 0;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.164330928;
  fExpectedDecLog = 0.012756021;
  fExpectedTipup = 0.049791591;
  fExpectedFFMoss = 0.270592511;
  fExpectedFFLitter = 0.502528949;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.27914526;
  fExpectedDecLog = 0.018642678;
  fExpectedTipup = 0.117849919;
  fExpectedFFMoss = 0.20452675;
  fExpectedFFLitter = 0.379835393;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.069011889;
  fExpectedDecLog = 0.119698936;
  fExpectedTipup = 0.035704405;
  fExpectedFFMoss = 0.271454669;
  fExpectedFFLitter = 0.5041301;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.090818511;
  fExpectedDecLog = 0.158779574;
  fExpectedTipup = 0.054255551;
  fExpectedFFMoss = 0.243651228;
  fExpectedFFLitter = 0.452495137;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0.000108295;
  fExpectedDecLog = 0.004330594;
  fExpectedTipup = 0.197261163;
  fExpectedFFMoss = 0.279404982;
  fExpectedFFLitter = 0.518894966;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS1Cohorts()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS1Cohorts(clGridBase *p_oSubstrateGrid) {
  clPackage * p_oCohort;
  float fExpectedTipup = 0.2, fExpectedFreshLog = 0.01, fExpectedScarSoil = 0.0, fActual;
  int iX, iY, iAge = 1, iActual;

  //Cell 1
  iX = 0; iY = 0;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 2
  iY = 1;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 3
  iY = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 4
  iY = 3;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 5
  iY = 4;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS2Cohorts()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS2Cohorts(clGridBase *p_oSubstrateGrid) {
  clPackage * p_oCohort;
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil = 0, fActual;
  int iX, iY, iAge, iActual;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedTipup = 0.2;
  fExpectedFreshLog = 0.01;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 2
  iY = 1;
  fExpectedTipup = 0.2;
  fExpectedFreshLog = 0.01;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 3 cohort 1
  iY = 2;
  fExpectedTipup = 0.172029331;
  fExpectedFreshLog = 0.008601467;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  //Cell 3 cohort 2
  fExpectedTipup = 0.023586489;
  fExpectedFreshLog = 0.116266855;
  iAge = 1;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());


  //Cell 4, cohort 1
  iY = 3;
  fExpectedTipup = 0.161187791;
  fExpectedFreshLog = 0.00805939;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  //Cell 4, cohort 2
  fExpectedTipup = 0.038328044;
  fExpectedFreshLog = 0.155733;
  iAge = 1;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 5
  iY = 4;
  fExpectedTipup = 0.2;
  fExpectedFreshLog = 0.01;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS3Cohorts()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS3Cohorts(clGridBase *p_oSubstrateGrid) {
  clPackage * p_oCohort;
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil = 0, fActual;
  int iX, iY, iAge, iActual;

  //Cell 1, cohort 1
  iX = 0; iY = 0;
  fExpectedTipup = 0.049826457;
  fExpectedFreshLog = 0.172756354;
  iAge = 1;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 2, cohort 1
  iY = 1;
  fExpectedTipup = 0.117932443;
  fExpectedFreshLog = 0.293457344;
  iAge = 1;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());


  //Cell 3, cohort 1
  iY = 2;
  fExpectedTipup = 0.021563079;
  fExpectedFreshLog = 0.106292692;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  //Cell 3, cohort 2
  fExpectedTipup = 0.014446724;
  fExpectedFreshLog = 0.071340092;
  iAge = 1;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 4, cohort 1
  iY = 3;
  fExpectedTipup = 0.033931489;
  fExpectedFreshLog = 0.137869092;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  //Cell 4, cohort 2
  fExpectedTipup = 0.020803283;
  fExpectedFreshLog = 0.093905275;
  iAge = 1;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 5
  iY = 4;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL == p_oCohort);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS4Cohorts()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS4Cohorts(clGridBase *p_oSubstrateGrid) {
  clPackage * p_oCohort;
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil = 0, fActual;
  int iX, iY, iAge, iActual;

  //Cell 1, cohort 1
  iX = 0; iY = 0;
  fExpectedTipup = 0.049826457;
  fExpectedFreshLog = 0.172756354;
  iAge = 2;

  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);
  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 2, cohort 1
  iY = 1;
  fExpectedTipup = 0.117932443;
  fExpectedFreshLog = 0.293457344;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 3, cohort 1
  iY = 2;
  fExpectedTipup = 0.014446724;
  fExpectedFreshLog = 0.071340092;
  iAge = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 4, cohort 1
  iY = 3;
  fExpectedTipup = 0.020803283;
  fExpectedFreshLog = 0.093905275;
  iAge = 2;

  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 5
  iY = 4;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL == p_oCohort);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS1Tipup()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS1Tipup(clGridBase *p_oCalcsGrid) {

  float fExpectedTipup = 0.2, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newtipup"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS2Tipup()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS2Tipup(clGridBase *p_oCalcsGrid) {

  float fExpectedTipup, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newtipup"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedTipup = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedTipup = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedTipup = 0.023586489;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedTipup = 0.038328044;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedTipup = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS3Tipup()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS3Tipup(clGridBase *p_oCalcsGrid) {

  float fExpectedTipup, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newtipup"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedTipup = 0.049826457;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedTipup = 0.117932443;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedTipup = 0.014446724;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedTipup = 0.020803283;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedTipup = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS4Tipup()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS4Tipup(clGridBase *p_oCalcsGrid) {

  float fExpectedTipup = 0.0, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newtipup"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS1SubstrateArrays()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS1SubstrateArrays(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedLog = 0.01, //all values are the same
        fActual;
  int *p_iLogCodes = new int[3],
       iX, iY, i;

  //Fresh log arrays
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  fExpectedLog = 0.0;
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Decayed log arrays
  fExpectedLog = 0.11;
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "declog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  fExpectedLog = 0.0;
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  delete[] p_iLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS2SubstrateArrays()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS2SubstrateArrays(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedLog, //all values are the same
        fActual;
  int *p_iLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.116266855;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.155733;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.01;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  fExpectedLog = 0;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Decayed log arrays
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "declog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  fExpectedLog = 0;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  delete[] p_iLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS3SubstrateArrays()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS3SubstrateArrays(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedLog, //all values are the same
        fActual;
  int *p_iLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.172756354;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.293457344;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.071340092;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.093905275;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.116266855;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.155733;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  fExpectedLog = 0.01;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Decayed log arrays
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "declog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.00940399;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.009403999;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.015074401;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.016999187;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.009403999;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  fExpectedLog = 0.11;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  delete[] p_iLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun1TS4SubstrateArrays()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun1TS4SubstrateArrays(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedLog, //all values are the same
        fActual;
  int *p_iLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  fExpectedLog = 0.0;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  iX = 0; iY = 0;
  fExpectedLog = 0.172756354;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.293457344;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.071340092;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.093905275;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.116266855;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.155733;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Decayed log arrays
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "declog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  fExpectedLog = 0.008425427;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  fExpectedLog = 0.014312084;
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  fExpectedLog = 0.112816642;
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  fExpectedLog = 0.151031119;
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  fExpectedLog = 0;
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  iX = 0; iY = 0;
  fExpectedLog = 0.009403999;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.009403999;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.015074401;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.016999187;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.009403999;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.000487706;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  delete[] p_iLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun4TS1Proportions()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun4TS1Proportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil,
        fExpectedDecLog, fExpectedFFMoss, fExpectedFFLitter, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedScarSoil = 0;
  fExpectedFreshLog = 0.01;
  fExpectedDecLog = 0.11;
  fExpectedTipup = 0.2;
  fExpectedFFMoss = 0.238;
  fExpectedFFLitter = 0.442;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iY = 1;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iY = 2;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iY = 3;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 5
  iY = 4;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun4TS2Proportions()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun4TS2Proportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil = 0,
        fExpectedDecLog, fExpectedFFMoss, fExpectedFFLitter, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.000964946;
  fExpectedDecLog = 0.019059036;
  fExpectedTipup = 0.199416888;
  fExpectedFFMoss = 0.273195696;
  fExpectedFFLitter = 0.507363435;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.000964946;
  fExpectedDecLog = 0.019059036;
  fExpectedTipup = 0.199416888;
  fExpectedFFLitter = 0.507363435;
  fExpectedFFMoss = 0.273195696;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.11709685;
  fExpectedDecLog = 0.019059036;
  fExpectedTipup = 0.195114258;
  fExpectedFFMoss = 0.23405545;
  fExpectedFFLitter = 0.434674407;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.156510687;
  fExpectedDecLog = 0.019059036;
  fExpectedTipup = 0.199045883;
  fExpectedFFMoss = 0.218884538;
  fExpectedFFLitter = 0.406499856;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedFreshLog = 8.52217E-09;
  fExpectedDecLog = 0.019059036;
  fExpectedTipup = 0.199416888;
  fExpectedFFMoss = 0.273533424;
  fExpectedFFLitter = 0.507990644;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun4TS3Proportions()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun4TS3Proportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil = 0,
        fExpectedDecLog, fExpectedFFMoss, fExpectedFFLitter, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.172756354;
  fExpectedDecLog = 0.00270174;
  fExpectedTipup = 0.204196066;
  fExpectedFFMoss = 0.217121044;
  fExpectedFFLitter = 0.403224796;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.293457344;
  fExpectedDecLog = 0.00270174;
  fExpectedTipup = 0.234811166;
  fExpectedFFMoss = 0.164160412;
  fExpectedFFLitter = 0.304869337;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.081596759;
  fExpectedDecLog = 0.107749476;
  fExpectedTipup = 0.192091779;
  fExpectedFFMoss = 0.216496695;
  fExpectedFFLitter = 0.402065291;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.107208893;
  fExpectedDecLog = 0.143407352;
  fExpectedTipup = 0.196311775;
  fExpectedFFMoss = 0.193575193;
  fExpectedFFLitter = 0.359496787;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedFreshLog = 3.24311E-14;
  fExpectedDecLog = 0.00270174;
  fExpectedTipup = 0.19856727;
  fExpectedFFMoss = 0.279555846;
  fExpectedFFLitter = 0.519175143;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun4TS4Proportions()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun4TS4Proportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedTipup, fExpectedFreshLog, fExpectedScarSoil,
        fExpectedDecLog, fExpectedFFMoss, fExpectedFFLitter, fActual;
  int iX, iY;

  fExpectedScarSoil = 0;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.016670049;
  fExpectedDecLog = 0.156249266;
  fExpectedTipup = 0.049681185;
  fExpectedFFMoss = 0.272089825;
  fExpectedFFLitter = 0.505309674;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.028317039;
  fExpectedDecLog = 0.265303266;
  fExpectedTipup = 0.117588604;
  fExpectedFFMoss = 0.206076882;
  fExpectedFFLitter = 0.382714209;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.006883931;
  fExpectedDecLog = 0.085410938;
  fExpectedTipup = 0.035813212;
  fExpectedFFMoss = 0.305162172;
  fExpectedFFLitter = 0.566729747;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.009061349;
  fExpectedDecLog = 0.11285637;
  fExpectedTipup = 0.054431046;
  fExpectedFFMoss = 0.288277932;
  fExpectedFFLitter = 0.535373303;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedFreshLog = 3.24311E-14;
  fExpectedDecLog = 0.000162961;
  fExpectedTipup = 0.19856727;
  fExpectedFFMoss = 0.280444419;
  fExpectedFFLitter = 0.52082535;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun4TS1SubstrateArrays()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun4TS1SubstrateArrays(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedLog = 0.01, //all values are the same
        fActual;
  int *p_iLogCodes = new int[3],
       iX, iY, i;

  //Fresh log arrays
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  fExpectedLog = 0.0;
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Decayed log arrays
  fExpectedLog = 0.11;
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "declog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  fExpectedLog = 0;
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  delete[] p_iLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun4TS2SubstrateArrays()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun4TS2SubstrateArrays(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedLog, //all values are the same
        fActual;
  int *p_iLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.116266855;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.155733;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.01;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  fExpectedLog = 0;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Decayed log arrays
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "declog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  delete[] p_iLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun4TS3SubstrateArrays()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun4TS3SubstrateArrays(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedLog, //all values are the same
        fActual;
  int *p_iLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.172756354;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.293457344;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.071340092;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.093905275;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.116266855;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.155733;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  fExpectedLog = 0.01;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Decayed log arrays
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "declog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.000964946;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.000964946;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.106012681;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.141670557;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.000964946;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.11;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  delete[] p_iLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyRun4TS4SubstrateArrays()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::VerifyRun4TS4SubstrateArrays(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedLog, //all values are the same
        fActual;
  int *p_iLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  fExpectedLog = 0.0;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  iX = 0; iY = 0;
  fExpectedLog = 0.172756354;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.293457344;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.071340092;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.093905275;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.116266855;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.155733;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Decayed log arrays
  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "declog_", i );
    p_iLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.156086305;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.265140305;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.075675281;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.099871314;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.000964946;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.000964946;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.106012681;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.141670557;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.000964946;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedLog = 0.009035054;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLog) < 0.001);

  delete[] p_iLogCodes;
}


/////////////////////////////////////////////////////////////////////////////
//  TestHarvestSubstrate()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::TestHarvestSubstrate(clSimManager *p_oSimManager)
{
  clTreePopulation *p_oPop;
  clGridBase * p_oSubstrateGrid;
  clPackage * p_oCohort;
  float fExpectedScarSoil, fExpectedFFMoss, fExpectedFFLitter, fExpectedTipup, fExpectedDecLog, fExpectedFreshLog, fActual;
  int iX, iY;

  //Feed file # 1 and run it
  p_oSimManager->ReadFile(WriteXMLFile1());
  p_oPop = (clTreePopulation*) p_oSimManager->GetPopulationObject("treepopulation");
  p_oSimManager->RunSim(1);
  p_oPop->CreateTree(1, 1, 0, clTreePopulation::adult, 12);
  p_oPop->CreateTree(1, 9, 0, clTreePopulation::adult, 12);
  p_oPop->CreateTree(1, 17, 0, clTreePopulation::adult, 12);

  p_oSimManager->RunSim(1);

  p_oSubstrateGrid = p_oSimManager->GetGridObject("Substrate");

  //Verify the substrates of the clear cut cell
  fExpectedScarSoil = 0.36;
  fExpectedFFMoss = 0.126;
  fExpectedFFLitter = 0.054;
  fExpectedTipup = 0.28;
  fExpectedDecLog = 0.01;
  fExpectedFreshLog = 0.17;
  iX = 0; iY = 1;

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  //There should be one cohort per cell which also matches the clear cut conditions
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Verify the substrates of the gap cut cell
  fExpectedScarSoil = 0.45;
  fExpectedFFMoss = 0.084;
  fExpectedFFLitter = 0.036;
  fExpectedTipup = 0.34;
  fExpectedDecLog = 0.0;
  fExpectedFreshLog = 0.09;
  iX = 0; iY = 0;

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  //There should be one cohort per cell which also matches the clear cut conditions
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Verify the substrates of the partial cut cell
  fExpectedScarSoil = 0.17;
  fExpectedFFMoss = 0.406;
  fExpectedFFLitter = 0.174;
  fExpectedTipup = 0.15;
  fExpectedDecLog = 0.04;
  fExpectedFreshLog = 0.06;
  iX = 0; iY = 2;

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

  //There should be one cohort per cell which also matches the clear cut conditions
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());
}

/////////////////////////////////////////////////////////////////////////////
//  TestSubstrateGridSetup()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::TestSubstrateGridSetup(clSimManager *p_oSimManager)
{
  clGridBase * p_oSubstrateGrid, *p_oSubstrateCalcsGrid;
  float fVal;

  //Feed file # 1 and make sure grid resolutions are right
  p_oSimManager->ReadFile(WriteXMLFile1());
  p_oSubstrateGrid = p_oSimManager->GetGridObject("Substrate");
  CPPUNIT_ASSERT(fabs(p_oSubstrateGrid->GetLengthXCells() - 8) < 0.001);
  CPPUNIT_ASSERT(fabs(p_oSubstrateGrid->GetLengthYCells() - 8) < 0.001);

  //Feed file # 2 and make sure grid resolutions are right
  p_oSimManager->ReadFile(WriteXMLFile5());
  p_oSubstrateGrid = p_oSimManager->GetGridObject("Substrate");
  CPPUNIT_ASSERT(fabs(p_oSubstrateGrid->GetLengthXCells() - 3) < 0.001);
  CPPUNIT_ASSERT(fabs(p_oSubstrateGrid->GetLengthYCells() - 2) < 0.001);
  //Make sure the substrate calcs resolutions are right and the values got read
  p_oSubstrateCalcsGrid = p_oSimManager->GetGridObject("substratecalcs");
  CPPUNIT_ASSERT(fabs(p_oSubstrateCalcsGrid->GetLengthXCells() - 3) < 0.001);
  CPPUNIT_ASSERT(fabs(p_oSubstrateCalcsGrid->GetLengthYCells() - 2) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("newtipup"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.1) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.01) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.02) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_2"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.03) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_3"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.04) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_4"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.05) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_5"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.06) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_6"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.07) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_7"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.08) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_8"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.09) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_9"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.11) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("declog_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.11) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("declog_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.12) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("declog_2"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.13) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("declog_3"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.14) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("declog_4"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.15) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("declog_5"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.16) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("declog_6"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.17) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("declog_7"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.18) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("declog_8"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.19) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("declog_9"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.20) < 0.001);

  //Feed file # 3 and make sure grid resolutions are right
  p_oSimManager->ReadFile(WriteXMLFile2());
  p_oSubstrateGrid = p_oSimManager->GetGridObject("Substrate");
  CPPUNIT_ASSERT(fabs(p_oSubstrateGrid->GetLengthXCells() - 8) < 0.001);
  CPPUNIT_ASSERT(fabs(p_oSubstrateGrid->GetLengthYCells() - 8) < 0.001);

  //Now check that initial conditions were correctly populated
  clPackage * p_oCohort;
  float fExpectedScarSoil = 0, fExpectedFFMoss = 0.238, fExpectedFFLitter = 0.442,
  fExpectedTipup = 0.2, fExpectedDecLog = 0.11,
       fExpectedFreshLog = 0.01, fActual;
  int iNumXCells = p_oSubstrateGrid->GetNumberXCells(), iNumYCells = p_oSubstrateGrid->GetNumberYCells(), iX, iY;

  for (iX = 0; iX < iNumXCells; iX++)
  {
    for (iY = 0; iY < iNumYCells; iY++)
    {

      //The values all should match the parameter file initial conditions
      p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("scarsoil"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

      p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("ffmoss"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedFFMoss) < 0.001);

      p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("fflitter"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedFFLitter) < 0.001);

      p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("tipup"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

      p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

      p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("declog"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedDecLog) < 0.001);

      //There should be one cohort per cell which also matches the initial conditions
      p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);

      p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("scarsoil"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedScarSoil) < 0.001);

      p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("tipup"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedTipup) < 0.001);

      p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

      CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());
    }
  }
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestSubstrate::WriteXMLFile1()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>4</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>";

  WriteCommonTreeStuff(oOut);

  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>harvest</behaviorName>"
       << "<version>1</version>"
       << "</behavior>"
       //Have to have a mortality behavior so dead is registered
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>substrate</behaviorName>"
       << "<version>2.1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       //Let the dead remover get rid of the trees we kill
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       << "<applyTo species=\"Species_1\" type=\"Sapling\" />"
       << "</behavior>"
       << "</behaviorList>"
       << "<harvest>"
       << "<ha_cutEvent>"
       << "<ha_applyToSpecies species=\"Species_1\" />"
       << "<ha_timestep>2</ha_timestep>"
       << "<ha_cutType>gap</ha_cutType>"
       << "<ha_cutAmountType>percent of density</ha_cutAmountType>"
       << "<ha_dbhRange>"
       << "<ha_low>0.0</ha_low>"
       << "<ha_high>25.0</ha_high>"
       << "<ha_amountToCut>100.0</ha_amountToCut>"
       << "</ha_dbhRange>"
       << "<ha_applyToCell x=\"0\" y=\"0\" />"
       << "</ha_cutEvent>"
       << "<ha_cutEvent>"
       << "<ha_applyToSpecies species=\"Species_1\" />"
       << "<ha_timestep>2</ha_timestep>"
       << "<ha_cutType>clear</ha_cutType>"
       << "<ha_cutAmountType>percent of density</ha_cutAmountType>"
       << "<ha_dbhRange>"
       << "<ha_low>0.0</ha_low>"
       << "<ha_high>25.0</ha_high>"
       << "<ha_amountToCut>100.0</ha_amountToCut>"
       << "</ha_dbhRange>"
       << "<ha_applyToCell x=\"0\" y=\"1\" />"
       << "</ha_cutEvent>"
       << "<ha_cutEvent>"
       << "<ha_applyToSpecies species=\"Species_1\" />"
       << "<ha_timestep>2</ha_timestep>"
       << "<ha_cutType>partial</ha_cutType>"
       << "<ha_cutAmountType>percent of density</ha_cutAmountType>"
       << "<ha_dbhRange>"
       << "<ha_low>0.0</ha_low>"
       << "<ha_high>15.0</ha_high>"
       << "<ha_amountToCut>100.0</ha_amountToCut>"
       << "</ha_dbhRange>"
       << "<ha_applyToCell x=\"0\" y=\"2\" />"
       << "</ha_cutEvent>"
       << "</harvest>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<su_scarSoilDecayAlpha>-5.1E-4</su_scarSoilDecayAlpha>"
       << "<su_scarSoilDecayBeta>4.4</su_scarSoilDecayBeta>"
       << "<su_tipupDecayAlpha>-7.0E-4</su_tipupDecayAlpha>"
       << "<su_tipupDecayBeta>4.3</su_tipupDecayBeta>"
       << "<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>"
       << "<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>"
       << "<su_decayedLogDecayAlpha>-0.7985</su_decayedLogDecayAlpha>"
       << "<su_decayedLogDecayBeta>1.1</su_decayedLogDecayBeta>"
       << "<su_maxNumberDecayYears>3</su_maxNumberDecayYears>"
       << "<su_initialScarSoil>0.0</su_initialScarSoil>"
       << "<su_initialTipup>0.2</su_initialTipup>"
       << "<su_initialFreshLog>0.01</su_initialFreshLog>"
       << "<su_initialDecayedLog>0.11</su_initialDecayedLog>"
       << "<su_partialCutScarSoil>0.17</su_partialCutScarSoil>"
       << "<su_partialCutTipup>0.15</su_partialCutTipup>"
       << "<su_partialCutFreshLog>0.06</su_partialCutFreshLog>"
       << "<su_partialCutDecayedLog>0.04</su_partialCutDecayedLog>"
       << "<su_gapCutScarSoil>0.45</su_gapCutScarSoil>"
       << "<su_gapCutTipup>0.34</su_gapCutTipup>"
       << "<su_gapCutFreshLog>0.09</su_gapCutFreshLog>"
       << "<su_gapCutDecayedLog>0.0</su_gapCutDecayedLog>"
       << "<su_clearCutScarSoil>0.36</su_clearCutScarSoil>"
       << "<su_clearCutTipup>0.28</su_clearCutTipup>"
       << "<su_clearCutFreshLog>0.17</su_clearCutFreshLog>"
       << "<su_clearCutDecayedLog>0.01</su_clearCutDecayedLog>"
       << "<su_propOfDeadFall>"
       << "<su_podfVal species=\"Species_1\">1</su_podfVal>"
       << "</su_propOfDeadFall>"
       << "<su_propOfFallUproot>"
       << "<su_pofuVal species=\"Species_1\">1</su_pofuVal>"
       << "</su_propOfFallUproot>"
       << "<su_propOfSnagsUproot>"
       << "<su_posuVal species=\"Species_1\">0</su_posuVal>"
       << "</su_propOfSnagsUproot>"
       << "<su_mossProportion>0.7</su_mossProportion>"
       << "<su_directionalTreeFall>0</su_directionalTreeFall>"
       << "<su_rootTipupFactor>3.1</su_rootTipupFactor>"
       << "</substrate>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestSubstrate::WriteXMLFile2()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>5</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>" << "<plot_lenY>200.0</plot_lenY>" << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>";

  WriteCommonTreeStuff(oOut);

  oOut << "<behaviorList>"
       //Have to have a mortality behavior so dead is registered
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>substrate</behaviorName>"
       << "<version>2.1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<su_scarSoilDecayAlpha>-5.1E-4</su_scarSoilDecayAlpha>"
       << "<su_scarSoilDecayBeta>4.4</su_scarSoilDecayBeta>"
       << "<su_tipupDecayAlpha>-7.0E-4</su_tipupDecayAlpha>"
       << "<su_tipupDecayBeta>4.3</su_tipupDecayBeta>"
       << "<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>"
       << "<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>"
       << "<su_decayedLogDecayAlpha>-0.7985</su_decayedLogDecayAlpha>"
       << "<su_decayedLogDecayBeta>1</su_decayedLogDecayBeta>"
       << "<su_maxNumberDecayYears>3</su_maxNumberDecayYears>"
       << "<su_initialScarSoil>0.0</su_initialScarSoil>"
       << "<su_initialTipup>0.2</su_initialTipup>"
       << "<su_initialFreshLog>0.01</su_initialFreshLog>"
       << "<su_initialDecayedLog>0.11</su_initialDecayedLog>"
       << "<su_propOfDeadFall>"
       << "<su_podfVal species=\"Species_1\">1</su_podfVal>"
       << "</su_propOfDeadFall>"
       << "<su_propOfFallUproot>"
       << "<su_pofuVal species=\"Species_1\">1</su_pofuVal>"
       << "</su_propOfFallUproot>"
       << "<su_propOfSnagsUproot>"
       << "<su_posuVal species=\"Species_1\">1</su_posuVal>"
       << "</su_propOfSnagsUproot>"
       << "<su_mossProportion>0.35</su_mossProportion>"
       << "<su_directionalTreeFall>0</su_directionalTreeFall>"
       << "<su_rootTipupFactor>3.1</su_rootTipupFactor>"
       << "</substrate>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile3()
/////////////////////////////////////////////////////////////////////////////
char * clTestSubstrate::WriteXMLFile3()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>5</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>";

  WriteCommonTreeStuff(oOut);

  oOut << "<behaviorList>"
       //Have to have a mortality behavior so dead is registered
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>substrate</behaviorName>"
       << "<version>2.1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">1.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<su_scarSoilDecayAlpha>-5.1E-4</su_scarSoilDecayAlpha>"
       << "<su_scarSoilDecayBeta>4.4</su_scarSoilDecayBeta>"
       << "<su_tipupDecayAlpha>-7.0E-4</su_tipupDecayAlpha>"
       << "<su_tipupDecayBeta>4.3</su_tipupDecayBeta>"
       << "<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>"
       << "<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>"
       << "<su_decayedLogDecayAlpha>-0.7985</su_decayedLogDecayAlpha>"
       << "<su_decayedLogDecayBeta>1</su_decayedLogDecayBeta>"
       << "<su_maxNumberDecayYears>10</su_maxNumberDecayYears>"
       << "<su_initialScarSoil>0.0</su_initialScarSoil>"
       << "<su_initialTipup>0.2</su_initialTipup>"
       << "<su_initialFreshLog>0.01</su_initialFreshLog>"
       << "<su_initialDecayedLog>0.11</su_initialDecayedLog>"
       << "<su_propOfDeadFall>"
       << "<su_podfVal species=\"Species_1\">1</su_podfVal>"
       << "</su_propOfDeadFall>"
       << "<su_propOfFallUproot>"
       << "<su_pofuVal species=\"Species_1\">1</su_pofuVal>"
       << "</su_propOfFallUproot>"
       << "<su_propOfSnagsUproot>"
       << "<su_posuVal species=\"Species_1\">1</su_posuVal>"
       << "</su_propOfSnagsUproot>"
       << "<su_mossProportion>0.35</su_mossProportion>"
       << "<su_directionalTreeFall>1</su_directionalTreeFall>"
       << "<su_rootTipupFactor>3.1</su_rootTipupFactor>" << "</substrate>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile4()
/////////////////////////////////////////////////////////////////////////////
char * clTestSubstrate::WriteXMLFile4()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>10</timesteps>"
       << "<yearsPerTimestep>3</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>270.0</plot_lenX>"
       << "<plot_lenY>270.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_3\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>"
       << "<grid gridName=\"Substrate\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"scarsoil\">0</ma_floatCode>"
       << "<ma_floatCode label=\"ffmoss\">1</ma_floatCode>"
       << "<ma_floatCode label=\"tipup\">2</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog\">3</ma_floatCode>"
       << "<ma_floatCode label=\"declog\">4</ma_floatCode>"
       << "<ma_floatCode label=\"fflitter\">5</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_lengthXCells>45</ma_lengthXCells>"
       << "<ma_lengthYCells>45</ma_lengthYCells>"
       << "</grid>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1.0</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_2\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_3\" type=\"Snag\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>substrate</behaviorName>"
       << "<version>2.1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_2\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_3\" type=\"Snag\"/>"
       << "</behavior>"
       //Let the dead remover get rid of the trees we kill
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_2\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_3\" type=\"Snag\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">1.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_2\">1.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_3\">1.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<su_scarSoilDecayAlpha>-5.1E-4</su_scarSoilDecayAlpha>"
       << "<su_scarSoilDecayBeta>4.4</su_scarSoilDecayBeta>"
       << "<su_tipupDecayAlpha>-7.0E-4</su_tipupDecayAlpha>"
       << "<su_tipupDecayBeta>4.3</su_tipupDecayBeta>"
       << "<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>"
       << "<su_freshLogDecayBeta>4.4</su_freshLogDecayBeta>"
       << "<su_decayedLogDecayAlpha>-0.7985</su_decayedLogDecayAlpha>"
       << "<su_decayedLogDecayBeta>1</su_decayedLogDecayBeta>"
       << "<su_maxNumberDecayYears>50</su_maxNumberDecayYears>"
       << "<su_initialScarSoil>0.0</su_initialScarSoil>"
       << "<su_initialTipup>0.2</su_initialTipup>"
       << "<su_initialFreshLog>0.01</su_initialFreshLog>"
       << "<su_initialDecayedLog>0.01</su_initialDecayedLog>"
       << "<su_partialCutScarSoil>0.17</su_partialCutScarSoil>"
       << "<su_partialCutTipup>0.15</su_partialCutTipup>"
       << "<su_partialCutFreshLog>0.06</su_partialCutFreshLog>"
       << "<su_partialCutDecayedLog>0.04</su_partialCutDecayedLog>"
       << "<su_gapCutScarSoil>0.45</su_gapCutScarSoil>"
       << "<su_gapCutTipup>0.34</su_gapCutTipup>"
       << "<su_gapCutFreshLog>0.09</su_gapCutFreshLog>"
       << "<su_gapCutDecayedLog>0.0</su_gapCutDecayedLog>"
       << "<su_clearCutScarSoil>0.36</su_clearCutScarSoil>"
       << "<su_clearCutTipup>0.34</su_clearCutTipup>"
       << "<su_clearCutFreshLog>0.17</su_clearCutFreshLog>"
       << "<su_clearCutDecayedLog>0.01</su_clearCutDecayedLog>"
       << "<su_propOfDeadFall>"
       << "<su_podfVal species=\"Species_1\">1</su_podfVal>"
       << "<su_podfVal species=\"Species_2\">1</su_podfVal>"
       << "<su_podfVal species=\"Species_3\">1</su_podfVal>"
       << "</su_propOfDeadFall>"
       << "<su_propOfFallUproot>"
       << "<su_pofuVal species=\"Species_1\">1</su_pofuVal>"
       << "<su_pofuVal species=\"Species_2\">0.5</su_pofuVal>"
       << "<su_pofuVal species=\"Species_3\">0.2</su_pofuVal>"
       << "</su_propOfFallUproot>"
       << "<su_propOfSnagsUproot>"
       << "<su_posuVal species=\"Species_1\">0.8</su_posuVal>"
       << "<su_posuVal species=\"Species_2\">0</su_posuVal>"
       << "<su_posuVal species=\"Species_3\">0.6</su_posuVal>"
       << "</su_propOfSnagsUproot>"
       << "<su_rootTipupFactor>3.1</su_rootTipupFactor>"
       << "<su_mossProportion>0</su_mossProportion>"
       << "<su_directionalTreeFall>0</su_directionalTreeFall>"
       << "</substrate>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile5()
/////////////////////////////////////////////////////////////////////////////
char * clTestSubstrate::WriteXMLFile5()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>10</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_3\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>"
       << "<grid gridName=\"Substrate\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"scarsoil\">0</ma_floatCode>"
       << "<ma_floatCode label=\"ffmoss\">1</ma_floatCode>"
       << "<ma_floatCode label=\"tipup\">2</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog\">3</ma_floatCode>"
       << "<ma_floatCode label=\"declog\">4</ma_floatCode>"
       << "<ma_floatCode label=\"fflitter\">5</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_lengthXCells>3</ma_lengthXCells>"
       << "<ma_lengthYCells>2</ma_lengthYCells>"
       << "</grid>"
       << "<grid gridName=\"substratecalcs\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"newtipup\">0</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_0\">1</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_1\">2</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_2\">3</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_3\">4</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_4\">5</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_5\">6</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_6\">7</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_7\">8</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_8\">9</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_9\">10</ma_floatCode>"
       << "<ma_floatCode label=\"declog_0\">11</ma_floatCode>"
       << "<ma_floatCode label=\"declog_1\">12</ma_floatCode>"
       << "<ma_floatCode label=\"declog_2\">13</ma_floatCode>"
       << "<ma_floatCode label=\"declog_3\">14</ma_floatCode>"
       << "<ma_floatCode label=\"declog_4\">15</ma_floatCode>"
       << "<ma_floatCode label=\"declog_5\">16</ma_floatCode>"
       << "<ma_floatCode label=\"declog_6\">17</ma_floatCode>"
       << "<ma_floatCode label=\"declog_7\">18</ma_floatCode>"
       << "<ma_floatCode label=\"declog_8\">19</ma_floatCode>"
       << "<ma_floatCode label=\"declog_9\">20</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_lengthXCells>3</ma_lengthXCells>"
       << "<ma_lengthYCells>2</ma_lengthYCells>"
       << "<ma_v x=\"0\" y=\"0\">"
       << "<fl c=\"0\">0.1</fl>"
       << "<fl c=\"1\">0.01</fl>"
       << "<fl c=\"2\">0.02</fl>"
       << "<fl c=\"3\">0.03</fl>"
       << "<fl c=\"4\">0.04</fl>"
       << "<fl c=\"5\">0.05</fl>"
       << "<fl c=\"6\">0.06</fl>"
       << "<fl c=\"7\">0.07</fl>"
       << "<fl c=\"8\">0.08</fl>"
       << "<fl c=\"9\">0.09</fl>"
       << "<fl c=\"10\">0.11</fl>"
       << "<fl c=\"11\">0.11</fl>"
       << "<fl c=\"12\">0.12</fl>"
       << "<fl c=\"13\">0.13</fl>"
       << "<fl c=\"14\">0.14</fl>"
       << "<fl c=\"15\">0.15</fl>"
       << "<fl c=\"16\">0.16</fl>"
       << "<fl c=\"17\">0.17</fl>"
       << "<fl c=\"18\">0.18</fl>"
       << "<fl c=\"19\">0.19</fl>"
       << "<fl c=\"20\">0.20</fl>"
       << "</ma_v>"
       << "</grid>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1.0</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>juvstochasticmort</behaviorName>"
       << "<version>1.0</version>"
       << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_2\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_3\" type=\"Snag\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>substrate</behaviorName>"
       << "<version>2.1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_2\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_3\" type=\"Snag\"/>"
       << "</behavior>"
       //Let the dead remover get rid of the trees we kill
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_2\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_3\" type=\"Snag\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">1.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_2\">1.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_3\">1.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "<mo_randomJuvenileMortality>"
       << "<mo_rjmVal species=\"Species_1\">0.0</mo_rjmVal>"
       << "<mo_rjmVal species=\"Species_2\">0.0</mo_rjmVal>"
       << "<mo_rjmVal species=\"Species_3\">0.0</mo_rjmVal>"
       << "</mo_randomJuvenileMortality>"
       << "</mortality>"
       << "<substrate>"
       << "<su_scarSoilDecayAlpha>-5.1E-4</su_scarSoilDecayAlpha>"
       << "<su_scarSoilDecayBeta>4.4</su_scarSoilDecayBeta>"
       << "<su_tipupDecayAlpha>-7.0E-4</su_tipupDecayAlpha>"
       << "<su_tipupDecayBeta>4.3</su_tipupDecayBeta>"
       << "<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>"
       << "<su_freshLogDecayBeta>4.4</su_freshLogDecayBeta>"
       << "<su_decayedLogDecayAlpha>-0.7985</su_decayedLogDecayAlpha>"
       << "<su_decayedLogDecayBeta>1.1</su_decayedLogDecayBeta>"
       << "<su_maxNumberDecayYears>10</su_maxNumberDecayYears>"
       << "<su_initialScarSoil>0.0</su_initialScarSoil>"
       << "<su_initialTipup>0.2</su_initialTipup>"
       << "<su_initialFreshLog>0.01</su_initialFreshLog>"
       << "<su_initialDecayedLog>0.01</su_initialDecayedLog>"
       << "<su_partialCutScarSoil>0.17</su_partialCutScarSoil>"
       << "<su_partialCutTipup>0.15</su_partialCutTipup>"
       << "<su_partialCutFreshLog>0.06</su_partialCutFreshLog>"
       << "<su_partialCutDecayedLog>0.04</su_partialCutDecayedLog>"
       << "<su_gapCutScarSoil>0.45</su_gapCutScarSoil>"
       << "<su_gapCutTipup>0.34</su_gapCutTipup>"
       << "<su_gapCutFreshLog>0.09</su_gapCutFreshLog>"
       << "<su_gapCutDecayedLog>0.0</su_gapCutDecayedLog>"
       << "<su_clearCutScarSoil>0.36</su_clearCutScarSoil>"
       << "<su_clearCutTipup>0.34</su_clearCutTipup>"
       << "<su_clearCutFreshLog>0.17</su_clearCutFreshLog>"
       << "<su_clearCutDecayedLog>0.01</su_clearCutDecayedLog>"
       << "<su_propOfDeadFall>"
       << "<su_podfVal species=\"Species_1\">0.2</su_podfVal>"
       << "<su_podfVal species=\"Species_2\">0.5</su_podfVal>"
       << "<su_podfVal species=\"Species_3\">0.0</su_podfVal>"
       << "</su_propOfDeadFall>"
       << "<su_propOfFallUproot>"
       << "<su_pofuVal species=\"Species_1\">1</su_pofuVal>"
       << "<su_pofuVal species=\"Species_2\">1</su_pofuVal>"
       << "<su_pofuVal species=\"Species_3\">1</su_pofuVal>"
       << "</su_propOfFallUproot>"
       << "<su_propOfSnagsUproot>"
       << "<su_posuVal species=\"Species_1\">1</su_posuVal>"
       << "<su_posuVal species=\"Species_2\">0.7</su_posuVal>"
       << "<su_posuVal species=\"Species_3\">0.15</su_posuVal>"
       << "</su_propOfSnagsUproot>"
       << "<su_rootTipupFactor>3.1</su_rootTipupFactor>"
       << "<su_mossProportion>0</su_mossProportion>"
       << "<su_directionalTreeFall>0</su_directionalTreeFall>"
       << "</substrate>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile6()
/////////////////////////////////////////////////////////////////////////////
char * clTestSubstrate::WriteXMLFile6()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>5</timesteps>"
       << "<yearsPerTimestep>3</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>";

  WriteCommonTreeStuff(oOut);

  oOut << "<behaviorList>"
       //Have to have a mortality behavior so dead is registered
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>substrate</behaviorName>"
       << "<version>2.1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<su_scarSoilDecayAlpha>-5.1E-4</su_scarSoilDecayAlpha>"
       << "<su_scarSoilDecayBeta>4.1</su_scarSoilDecayBeta>"
       << "<su_tipupDecayAlpha>-7.0E-4</su_tipupDecayAlpha>"
       << "<su_tipupDecayBeta>1.3</su_tipupDecayBeta>"
       << "<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>"
       << "<su_freshLogDecayBeta>3.5</su_freshLogDecayBeta>"
       << "<su_decayedLogDecayAlpha>-0.7985</su_decayedLogDecayAlpha>"
       << "<su_decayedLogDecayBeta>1</su_decayedLogDecayBeta>"
       << "<su_maxNumberDecayYears>9</su_maxNumberDecayYears>"
       << "<su_initialScarSoil>0.0</su_initialScarSoil>"
       << "<su_initialTipup>0.2</su_initialTipup>"
       << "<su_initialFreshLog>0.01</su_initialFreshLog>"
       << "<su_initialDecayedLog>0.11</su_initialDecayedLog>"
       << "<su_propOfDeadFall>"
       << "<su_podfVal species=\"Species_1\">1</su_podfVal>"
       << "</su_propOfDeadFall>"
       << "<su_propOfFallUproot>"
       << "<su_pofuVal species=\"Species_1\">1</su_pofuVal>"
       << "</su_propOfFallUproot>"
       << "<su_propOfSnagsUproot>"
       << "<su_posuVal species=\"Species_1\">1</su_posuVal>"
       << "</su_propOfSnagsUproot>"
       << "<su_mossProportion>0.35</su_mossProportion>"
       << "<su_directionalTreeFall>0</su_directionalTreeFall>"
       << "<su_rootTipupFactor>3.1</su_rootTipupFactor>"
       << "</substrate>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteCommonTreeStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestSubstrate::WriteCommonTreeStuff( std::fstream &oOut )
{
  using namespace std;
  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>";
}
